/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUI_CONDITIONAL_BEHAVIOUR__H_
#define GUI_CONDITIONAL_BEHAVIOUR__H_

#include "GUICmdBehaviour.h"

/** @brief Behaviour for repeating clicks on object
    Additional phases can be defined where the start-time of the phase is measured from the beginning of the previous phase
*/
class CGUIConditionalBehaviour : public CGUICmdBehaviour
{
public:
    /// Operations for behaviour
    enum Operation_t
    {
        OP_LESS, ///< less (<)
        OP_LESS_EQUAL, ///< less or equal (<=)
        OP_EQUAL, ///< equal (==)
        OP_NOT_EQUAL, ///< not equal (!=)
        OP_GREATER_EQUAL, ///< greater or equal (>=)
        OP_GREATER ///< greater (>)
    };

public:
    ///Default constructor
    CGUIConditionalBehaviour();

    virtual ~CGUIConditionalBehaviour();

    /** set the connected datapool
    @param eID
    */
    void SetDataPoolID(const DataPoolResource_t& eID);

    /** get connected datapool
    @return id
    */
    DataPoolResource_t GetDataPoolID() const;

    /** set index for datapool
    @param uiX
    @param uiY
    */
    void SetIndex(const eC_UInt& uiX, const eC_UInt& uiY);

    /** get index for datapool
    @param uiX
    @param uiY
    */
    void GetIndex(eC_UInt& uiX, eC_UInt& uiY) const;

    /** set the compare operation
    @param eOperation
    */
    void SetOperation(const Operation_t& eOperation);

    /** get the compare operation
    @return operation
    */
    Operation_t GetOperation() const;

    /** set compare-value
    the value will be converted to match the datapool-value
    @param kValue
    */
    void SetValue(const CGUIValue& kValue);

    /** get compare-value
    @return value
    */
    CGUIValue GetValue() const;

    /** set the command which is executed on true
    @param pkCmd
    */
    void SetCommand(CGUICommandPtr pkCmd);

    /** get the command which is executed on true
    @return command
    */
    CGUICommandPtr GetCommand() const;

#if defined(GUILIANI_STREAM_GUI)
    virtual void ReadFromStream();
#endif

#if defined(GUILIANI_WRITE_GUI)
    virtual void WriteToStream(const eC_Bool bWriteClassID = false);
#endif

protected:
    /**
        Executes the command for the given behaviour, if it is equal
        to the stored one.
        @param eFromBehaviour Behaviour slot from which this method was called.
    */
    virtual void ExecuteCommand(const BehaviourType_t& eFromBehaviour);

private:
    /** Copy-constructor.
        @param kSource Source object to be copied.
    */
    CGUIConditionalBehaviour(const CGUIConditionalBehaviour& kSource);

    /** Operator= method.
        @param kSource Source object to be copied.
        @return This instance.
    */
    CGUIConditionalBehaviour& operator=(const CGUIConditionalBehaviour& kSource);

private:
    DataPoolResource_t m_eDataPoolID; ///< id of datapool-resource
    eC_UInt m_uiX; ///< x-coordinate
    eC_UInt m_uiY; ///< y-coordinate

    Operation_t m_eOperation; ///< operation
    CGUIValue m_kValue; ///< value

    /// Stores the command executed on true
    CGUICommandPtr m_pkCommand;
};

#endif
