/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUI_MULTITOUCH_BEHAVIOUR_H__
#define __GUI_MULTITOUCH_BEHAVIOUR_H__

#include "GUIBehaviourDecorator.h"
#include "GUIObjectHandleResource.h"

/// @brief Behaviour for processing object state changes
class CGUIMultitouchBehaviour : public CGUIBehaviourDecorator
{
public:
    /// enum for gesture type
    enum MultitouchType_t
    {
        MT_SCALE, ///< scaling
        MT_SCROLL ///< scrolling
    };

public:
    /** Constructor
        @param eType type of gesture
        @param bKeepCenter if true the center of the object will be retained
        @param vFactor factor for gesture
    */
    CGUIMultitouchBehaviour(
        const MultitouchType_t& eType,
        const eC_Bool& bKeepCenter,
        const eC_Value& vFactor);

    CGUIMultitouchBehaviour();

#if defined(GUILIANI_STREAM_GUI)
    /** Reads all attributes from streaming file.
        This method is called by CGUIFactoryManager after one of the registered
        factories has created an instance of this class.
    */
    virtual void ReadFromStream();
#endif

#if defined(GUILIANI_WRITE_GUI)
    /** Writes all attributes to the streaming file. A CGUIStreamWriter
        has to be initialized first.
        @param bWriteClassID This flag is used to select if writing of command
               class ID, leading and trailing tags is performed.
    */
    virtual void WriteToStream(const eC_Bool bWriteClassID=false);
#endif

    /** Set the type of processing, either scaling or scrolling
    @param eType scaling or scrolling
    */
    void SetType(const MultitouchType_t& eType);

    /** Get the current set type of processing
    @return the type
    */
    MultitouchType_t GetType() const;

    /** Set the keep center position when resizing the object
    @param bKeepCenter if true the object will keep its center position
    */
    void SetKeepCenter(const eC_Bool& bKeepCenter);

    /** Get if the object should keep its center
    @return true if center will be kept
    */
    eC_Bool GetKeepCenter() const;

    /** Set the factor for scaling and scrolling
    @param vFactor the factor used for scaling and scrolling
    for scaling this factor will be multiplied with the delta movement
    for scrolling this factor is the number of pixels the object is moved
    */
    void SetFactor(const eC_Value& vFactor);

    /** Get the factor used for scaling and scrolling
    @return factor
    */
    eC_Value GetFactor() const;

protected:
    virtual eC_Bool DoScale(const eC_Value& vDelta);

    virtual eC_Bool DoScroll(
        const eC_Value& vAbsX,
        const eC_Value& vAbsY,
        const eC_Value& vDelta,
        const eC_Bool& bModifierActive);

private:
    MultitouchType_t m_eType;
    eC_Bool m_bKeepCenter;
    eC_Value m_vFactor;
};

#endif
