/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef CGUISHAPE__H_
#define CGUISHAPE__H_

#include "GUIRadialBar.h"

#include "GUIImageResource.h"
#include "GUICommonEnums.h"
#include "GUINinePatch.h"

/// A primitive vector shape with transformations

// @guiliani_doxygen toplevel_control Shape
/**
    <table border="0">
        <tr>
            <td width="200">@image html shape.png</td>
            <td>
                The "Shape" control is used to draw primitives with multiple endpoints.
                It is also possible to apply transformations.
            </td>
        </tr>
    </table>
*/
// @endguiliani_doxygen
/**
    @ingroup GUILIANI_CONTROLS
*/
class CGUIShape : public CGUIObject, public CGUIColorPropertyObserver
{
public:
    /** Constructor.
        @param pkParent Pointer to parent object.
        @param vX X position relative to its parent object.
        @param vY Y position relative to its parent object.
        @param vWidth Width of the control.
        @param vHeight Height of the control.
        @param eID object handle for the control.
    */
    CGUIShape(
        CGUICompositeObject* const pkParent,
        const eC_Value& vX,
        const eC_Value& vY,
        const eC_Value& vWidth,
        const eC_Value& vHeight,
        const ObjectHandle_t &eID = NO_HANDLE);

    /** Copy-constructor of the CGUIRadialProgressBar class.
        @param kSource Source object to be copied.
    */
    CGUIShape(const CGUIShape& kSource);

    /** Operator= method of the CGUIRadialProgressBar class.
        @param kSource Source object to be copied.
        @return Copied object.
    */
    CGUIShape& operator=(const CGUIShape& kSource);

    ///    Destructor
    virtual ~CGUIShape();

    virtual eC_Bool DoDraw();

#ifdef GUILIANI_STREAM_GUI
    CGUIShape();

    virtual void ReadFromStream();
#endif

#ifdef GUILIANI_WRITE_GUI
    virtual void WriteToStream(const eC_Bool bWriteClassID = false);
#endif

    /** return fill color
    @return fill color
    */
    eC_UInt GetColor() const;

    /** set fill color
    @param uiColor
    */
    void SetColor(const eC_UInt& uiColor);

    /** set fill color
    @param eColor
    */
    void SetColor(const GlobalProperty_t& eColor);

    /** return border color
    @return border color
    */
    eC_UInt GetBorderColor() const;

    /** set border color
    @param uiBorderColor
    */
    void SetBorderColor(const eC_UInt& uiBorderColor);

    /** set border color
    @param eBorderColor
    */
    void SetBorderColor(const GlobalProperty_t& eBorderColor);

    /** set line width
    @param vLineWidth
    */
    void SetLineWidth(const eC_Value& vLineWidth);

    /** return line width
    @return line width
    */
    eC_Value GetLineWidth() const;

    /** set relative coordinates mode
    @param bRelative
    */
    void SetRelative(const eC_Bool& bRelative);

    /** return if relative coordinates mode is active
    @return true if relative coordinates mode is active
    */
    eC_Bool GetRelative() const;

    /** clear all points
    */
    void ClearPoints();

    /** add a new point
    @param kPoint
    */
    void AddPoint(CGUIPoint kPoint);

    /** set if filled
    @param bFilled
    */
    void SetFilled(const eC_Bool& bFilled);

    /** return if filled
    @return true if filled
    */
    eC_Bool GetFilled() const;

    /** set transformation active
    @param bTransformActive
    */
    void SetTransformActive(const eC_Bool& bTransformActive);

    /** return if transformation is active
    @return true if transformation is active
    */
    eC_Bool GetTransformActive() const;

    /** set scale-factor
    @param kScale
    */
    void SetScale(const CGUIPoint& kScale);

    /** return scale-factor
    @return scale-factor
    */
    CGUIPoint GetScale() const;

    /** set rotation-angle
    @param vRotation
    */
    void SetRotation(const eC_Value& vRotation);

    /** return rotation-angle
    @return rotation-angle
    */
    eC_Value GetRotation() const;

    /** set translate-factor
    @param kTranslate
    */
    void SetTranslate(const CGUIPoint& kTranslate);

    /** return translate-factor
    @return translate-factor
    */
    CGUIPoint GetTranslate() const;

private:
    /** Help function to initialize the control.
    */
    void Init();

    /** Copy attributes from source control to this control.
        @param kSource Source object to be copied.
    */
    void CopyAttributes(const CGUIShape& kSource);

    /// Deinitialize the control and free the resources.
    void DeInit();

    void UpdatePoints(const eC_Bool& bSizeChanged);

private:
    static const eC_UInt INDEX_BORDERCOLOR;
    static const eC_UInt INDEX_COLOR;

    eC_Value m_vLineWidth;
    eC_Bool m_bRelative;
    eC_TArray<CGUIPoint> m_kPoints;

    eC_Bool m_bFilled;

    eC_Bool m_bTransformActive;
    CGUIPoint m_kScale;
    eC_Value m_vRotation;
    CGUIPoint m_kTranslate;

    eC_UInt m_uiDrawCount;
    CGUIPoint* m_pkDrawPoints;
};
#endif
