/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __TRIANGULATE_H__
#define __TRIANGULATE_H__

#include "eC_Types.h"
#include "eC_Math.h"

#include "GUIPoint.h"

/** class for twodimensional vectors
*/
class Vector2
{
public:
    /** default constructor
    */
    Vector2() :
        x(eC_FromInt(0)),
        y(eC_FromInt(0))
    {}

    /** constructor
    @param vX
    @param vY
    */
    Vector2(const eC_Value& vX, const eC_Value& vY) :
        x(vX),
        y(vY)
    {}

    /** addition operator
    @param rightVec
    @return result
    */
    Vector2 operator+(Vector2 rightVec) const
    {
        return Vector2(*this) += rightVec;
    }

    /** subtraction operator
    @param rightVec
    @return result
    */
    Vector2 operator-(Vector2 rightVec) const
    {
        return Vector2(*this) -= rightVec;
    }

    /** multiply operator
    @param scalar
    @return result
    */
    Vector2 operator*(float scalar) const
    {
        return Vector2(*this) *= scalar;
    }

    /** division operator
    @param scalar
    @return result
    */
    Vector2 operator/(float scalar) const
    {
        return Vector2(*this) /= scalar;
    }

    /** addition operator
    @param rightVec
    @return result
    */
    Vector2& operator+=(Vector2 rightVec)
    {
        x += rightVec.x;
        y += rightVec.y;
        return *this;
    }

    /** subtraction operator
    @param rightVec
    @return result
    */
    Vector2& operator-=(Vector2 rightVec)
    {
        x -= rightVec.x;
        y -= rightVec.y;
        return *this;
    }

    /** multiply operator
    @param scalar
    @return result
    */
    Vector2& operator*=(float scalar)
    {
        x = eC_Mul(x, scalar);
        y = eC_Mul(y, scalar);
        return *this;
    }

    /** division operator
    @param scalar
    @return result
    */
    Vector2& operator/=(float scalar)
    {
        x = eC_Div(x, scalar);
        y = eC_Div(y, scalar);
        return *this;
    }

    /** assignment operator
    @param rightVec
    @return result
    */
    Vector2& operator=(Vector2 rightVec)
    {
        x = rightVec.x;
        y = rightVec.y;
        return *this;
    }

    /** equal operator
    @param rightVec
    @return true if equal
    */
    bool operator==(const Vector2& rightVec) const
    {
        return (x == rightVec.x && y == rightVec.y);
    }

    /** not equal operator
    @param rightVec
    @return true if not equal
    */
    bool operator!=(const Vector2& rightVec) const
    {
        return !operator==(rightVec);
    }

    eC_Value x; ///< x-coordinate
    eC_Value y; ///< y-coordinate
};

/** helper class using ear-clipping algorithm
*/
class Triangulate
{
public:
    /** winding order of the resulting triangles
    */
    enum WindingOrder
    {
        CW = 1, ///< clockwise
        CCW = 2 ///< counter-clockwise
    };

public:
    /** run triangulation
    @param pvPoints
    @param uiCount
    @param eOrder
    @param puiTriangleIndices
    @param uiTriCount
    @return true if successful
    */
    static eC_Bool Run(
        eC_Value* pvPoints,
        const eC_UInt& uiCount,
        const WindingOrder& eOrder,
        eC_UInt** puiTriangleIndices,
        eC_UInt& uiTriCount);
};

#endif
