/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef CGFXWRAPBLU_H
#define CGFXWRAPBLU_H

#include "GfxWrap.h"
#include "GUIImageData.h"

#include "blu.h"

/// BLU-specific image-data
class CGUIImageDataBLU : public CGUIImageData
{
public:
    /// Constructor
    CGUIImageDataBLU() :
        CGUIImageData(),
        m_uiRGBSize(0),
        m_uiAlphaSize(0)
    {}

    virtual ~CGUIImageDataBLU() {}

    /** Set the RGB-size
    @param uiRGBSize
    */
    void SetRGBSize(const eC_UInt& uiRGBSize) { m_uiRGBSize = uiRGBSize; }
    
    /** Get RGB-size
    @return RGB-size
    */
    eC_UInt GetRGBSize() const { return m_uiRGBSize; }

    /** Set Alpha-size
    @param uiAlphaSize alpha-size
    */
    void SetAlphaSize(const eC_UInt& uiAlphaSize) { m_uiAlphaSize = uiAlphaSize; }
    
    /** Get Alpha-size
    @return alpha-size
    */
    eC_UInt GetAlphaSize() const { return m_uiAlphaSize; }

    /** Set byte per pixel
    @param ubBPP bytes per pixel
    */
    void SetBytePerPixel(const eC_UByte& ubBPP) { m_ubBPP = ubBPP; }
    
    /** Get bytes per pixel
    @return BPP
    */
    eC_UByte GetBytePerPixel() const { return m_ubBPP; }

private:
    eC_UInt m_uiRGBSize; ///< RGB-size
    eC_UInt m_uiAlphaSize; ///< alpha-size
    eC_UByte m_ubBPP; ///< bytes per pixel
};

/**
@brief BLU-specific graphics-wrapper
*/
class CGfxWrapBLU : public CGfxWrap
{
public:
    /// signature for function to set the framebuffer
    typedef void* (SetFrameBuffer(void*, unsigned long, unsigned long));

public:
    /**
    Singleton for creating a GfxWrap-Null object and change the pointer in
    base class to this gfx wrapper.
    @param pkGfxEnv graphics-environment
    @return true if successful
    */
    static eC_Bool CreateInstance(CGfxEnv* pkGfxEnv = NULL);

    /** Initialize */
    void Init();

    /** Initialize
    @param uiWidth
    @param uiHeight
    @param ubBitPerPixel
    @param uiNumberOfBuffers
    @param pkFrameBuffers
    */
    void Init(
        const eC_UInt &uiWidth,
        const eC_UInt &uiHeight,
        const eC_UByte &ubBitPerPixel,
        const eC_UInt uiNumberOfBuffers,
        void** pkFrameBuffers);

    virtual eC_Bool Refresh(const RefreshCall_t eRefreshCallType);

    virtual void SetCliprect(const CGUIRect &NewClipRect);

    virtual eC_Value SetLineWidth(const eC_Value& vWidth);

    virtual void Line(
        const eC_Value& vAbsX1, const eC_Value& vAbsY1,
        const eC_Value& vAbsX2, const eC_Value& vAbsY2);

    virtual void Rect(
        const eC_Value& vAbsX1, const eC_Value& vAbsY1,
        const eC_Value& vAbsX2, const eC_Value& vAbsY2);

    virtual void FilledRect(
        const eC_Value& vAbsX1, const eC_Value& vAbsY1,
        const eC_Value& vAbsX2, const eC_Value& vAbsY2);

    virtual void Ellipse(
        const eC_Value& vAbsX1, const eC_Value& vAbsY1,
        const eC_Value& vAbsX2, const eC_Value& vAbsY2,
        const eC_Bool &bFilled);

    virtual void Arc(
        const eC_Value& vX1, const eC_Value& vY1,
        const eC_Value& vX2, const eC_Value& vY2,
        const eC_Value& vStartAngle, const eC_Value& vEndAngle,
        const eC_Value& vRotationAngle,
        const eC_Bool &bFilled);

    virtual void Ring(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Value& vRingWidth,
        const eC_Value& vStartAngle,
        const eC_Value& vEndAngle,
        const eC_Value& vRotationAngle,
        const eC_Bool & bFilled);

    virtual void Polygon(
        eC_Value* pvPoints,
        const eC_UInt& uiCount,
        const eC_Bool& bFilled);

    virtual void SetForegroundColorImpl(
        const eC_UByte &ubRed,
        const eC_UByte &ubGreen,
        const eC_UByte &ubBlue,
        const eC_UByte &ubAlpha = 255);

    virtual eC_UInt GetImgWidthImpl(const ImageResource_t &eID) const;

    virtual eC_UInt GetImgHeightImpl(const ImageResource_t &eID) const;

    virtual eC_UInt GetImageSize(const ImageResource_t &eID) const;

    virtual void BlitImgExtImpl(
        const ImageResource_t &eID,
        const eC_UInt &uiAbsSrcX, const eC_UInt &uiAbsSrcY,
        const eC_UInt &uiSrcWidth, const eC_UInt &uiSrcHeight,
        const eC_Value& vAbsDestX, const eC_Value& vAbsDestY,
        const eC_Value& vDestWidth, const eC_Value& vDestHeight,
        const eC_UByte &ubAlpha = 255,
        const eC_Value& vAngle = eC_FromFloat(0),
        const eC_Value& vRotCenterX = eC_FromFloat(0.5), const eC_Value& vRotCenterY = eC_FromFloat(0.5));

    virtual void BlitImgExtImpl(const CGUIImageData& kImageData, const eC_Value& vAbsDestX, const eC_Value& vAbsDestY);

    virtual void LoadImgImpl(const eC_String& kPath, const ImageResource_t &eID);

    virtual void LoadImgImpl(
        void* pkMemory,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& eImageType,
        const ImageResource_t &eID);

    virtual void UnloadImg(const ImageResource_t &eImageID);

    virtual eC_Bool ImageExists(const ImageResource_t eID) const;

    virtual eC_UInt GetPhysicalScreenWidth();

    virtual eC_UInt GetPhysicalScreenHeight();

    virtual eC_UInt GetVirtualScreenWidth();

    virtual eC_UInt GetVirtualScreenHeight();

    virtual eC_Bool SupportCompressedFonts() const { return true; }

    /** Indicate if scratch-buffer is in use
    @return true if so
    */
    eC_Bool IsScratchBufferIsUse() const { return m_bScratchbufferInUse; }

    /** Get rect of scratch-buffer
    @return rect of scratch-buffer
    */
    CGUIRect GetScratchBufferRect() const { return m_kScratchbufferRect; }

    /** Return the destination-bitmap
    @return destination-bitmap
    */
    __blu_bitmap* GetDestination() { return &m_pkDestinationBitmaps[m_uiCurrentBuffer]; }

    virtual eC_Bool DrawToBitmap(GUIBitmapPlaneID_t uiBitmapID);
    virtual eC_Bool DrawToScreen();

    CGUIBitmapPlane* CreateGfxDependentBitmap(
        GUIBitmapPlaneID_t uiNewBitmapID,
        const CGUIObject& rkParentObject,
        const eC_Value& vWidth, const eC_Value& vHeight);

    /** Set the function which is called when the framebuffer will be flipped
    @param pkSetFrameBuffer
    */
    void SetFrameBufferFP(SetFrameBuffer pkSetFrameBuffer);

    /** Get the currently used threshold for alpha-blending (default: 127)
    @return threshold
    */
    eC_UByte GetAlphaThreshold() const { return m_ubAlphaThreshold; }

    /** Sets the threshold for alpha-blending. values below will be treated as fully transparent
    @param ubAlphaThreshold
    */
    void SetAlphaThreshold(const eC_UByte& ubAlphaThreshold) { m_ubAlphaThreshold = ubAlphaThreshold; }

protected:
    virtual eC_Bool SetNOFImagesImpl(const eC_UInt uiNOFImages);

    virtual void StartHandleDraw(const CGUIRect &crkClipRect);

    /** Constructor
    @param pkGfxEnv
    */
    CGfxWrapBLU(CGfxEnv* pkGfxEnv = NULL);

    /// Destructor
    virtual ~CGfxWrapBLU();

protected:
    /// Rect describing the scratchpad's position on the screen
    CGUIRect m_kScratchbufferRect;

    /// Whether we're using the scratch buffer instead of the display
    eC_Bool m_bScratchbufferInUse;

    __blu_bitmap* m_pkDestinationBitmaps; ///< destination-bitmaps
    __blu_bitmap m_kSourceBitmap; ///< source-bitmap
    __blu_bitmap m_kScratchPad; ///< scratch-buffer

    eC_UByte m_ubAlphaThreshold; ///< alpha-threshold
    eC_UInt m_uiSizeOfScratchpad; ///< size of sratch-buffer

    eC_TArray<CGUIImageDataBLU*> m_pkImages; ///< internal images

    eC_UInt m_uiNumberOfBuffers; ///< number of framebuffers used
    eC_UInt m_uiCurrentBuffer; ///< currently active framebuffer

    SetFrameBuffer* m_pkSetFrameBuffer; ///< function-pointer to specific flip-method

    static const eC_UInt cuiSegmentAngle; ///< angle of segment for arc and ring

private:
    /** Copy-constructor.
        @param kSource Source object to be copied.
    */
    CGfxWrapBLU(const CGfxWrapBLU& kSource);

    /** Operator= method.
        @param kSource Source object to be copied.
        @return This instance.
    */
    CGfxWrapBLU& operator=(const CGfxWrapBLU& kSource);

    eC_Bool LoadImgDecoder(const eC_String& kPath, const eC_UInt& uiImageIndex);
    eC_Bool LoadImgBLU(eC_File* pkImageFile, CGUIImageDataBLU& kImageData);
    eC_Bool LoadImgRAW(eC_File* pkImageFile, CGUIImageDataBLU& kImageData);

    /** Fill a polygon
    @param pkPoints points
    @param uiCount number of points
    @param ubMode mode
    */
    void FillPolygon(blu_point16_t* pkPoints, eC_UInt uiCount, eC_UByte ubMode);

    /** Draw a polygon
    @param pkPoints
    @param uiCount number of points
    */
    void DrawPolygon(blu_point16_t* pkPoints, eC_UInt uiCount);
};

#endif /* CGFXWRAPBLU_H */
