/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FNT_WRAP_FREETYPE_DAVE__H__
#define __FNT_WRAP_FREETYPE_DAVE__H__

#include "FontCacheFreeType.h"

//FreeType includes
#include "ft2build.h"
#include FT_GLYPH_H

#include "FntWrapFreeType.h"

/** This is the FreeType implementation of the Font wrapper for DAVE.

it will use a glyph-cache for each loaded font. this cache will use
DAVE-memory if necessary.
*/

/**
This is the FreeType implementation of the font wrapper for DAVE.
It implements the text rendering using DAVE.
For details see base class.
*/

class CFntWrapFreeTypeDAVE : public CFntWrapFreeType
{
public:
    /** Create a FntWrap-FreeType object. It should be created
    in implementation after creation of GfxWrap.
    @param uiCacheSize The max size of the freetype cache in bytes.
    @param uiMaxNumFaces Maximum number of opened font face objects managed by the FreeType cache.
    @param uiMaxNumSizeObjects Maximum number of opened size objects managed by the FreeType cache.
    @param uiGlyphCacheInitialSize initial size for glyph-cache
    @param bGlyphCacheAutoResize if true the cache will be resized if necessary
    @param uiGlyphCacheResizeBy amount of glyph to be resized
    @return True if it is successfully created, False otherwise.
    */
    static eC_Bool CreateInstance(
        const eC_UInt& uiCacheSize,
        const eC_UInt& uiMaxNumFaces = 10,
        const eC_UInt& uiMaxNumSizeObjects = 10,
        const eC_UInt& uiGlyphCacheInitialSize = 32,
        const eC_Bool& bGlyphCacheAutoResize = true,
        const eC_UInt& uiGlyphCacheResizeBy = 10);

protected:
    /** Create a FntWrap-FreeType object. It should be created
    in implementation after creation of GfxWrap.
    @param uiCacheSize The max size of the freetype cache in bytes.
    @param uiMaxNumFaces Maximum number of opened font face objects managed by the FreeType cache.
    @param uiMaxNumSizeObjects Maximum number of opened size objects managed by the FreeType cache.
    @param uiGlyphCacheInitialSize initial size for glyph-cache
    @param bGlyphCacheAutoResize if true the cache will be resized if necessary
    @param uiGlyphCacheResizeBy amount of glyph to be resized
    */
    CFntWrapFreeTypeDAVE(
        const eC_UInt& uiCacheSize,
        const eC_UInt& uiMaxNumFaces,
        const eC_UInt& uiMaxNumSizeObjects,
        const eC_UInt& uiGlyphCacheInitialSize,
        const eC_Bool& bGlyphCacheAutoResize,
        const eC_UInt& uiGlyphCacheResizeBy);

    virtual ~CFntWrapFreeTypeDAVE();

    virtual void LoadFont(const FontResource_t &eFontID);
    virtual void UnloadFont(const FontResource_t &eFontID);
    virtual void SetFont(const FontResource_t &eFontID);

    virtual void GetGlyphMetrics(const eC_Char cChar, GlyphMetrics_t& kMetrics);

    virtual eC_Int GetAscender() const;
    virtual eC_Int GetDescender() const;
    virtual eC_Int GetInternalLeading() const;

    eC_Bool SetNOFFonts(const eC_UInt uiNOFFonts);

    /** Glyph-cache used for faster access to glyphs.
    Also used to support memory allocation directly via D/AVE
    */
    eC_TArray<CFontCacheFreeType*> m_apFontCache;

private:
    void RenderTextInternal(
        const eC_String *lpString,
        const TextOperation_t &eOperation,
        const eC_Value& vX1,
        const eC_Value& vY1,
        eC_Value& vWidth,
        eC_Value& vHeight,
        eC_Value vWidthMax,
        eC_UInt &uiNumFittingChars);

    /** Renders a glyph to the screen.
    This function takes the 8 bit alpha image provided by freetype adds color and blits it to the screen.
    @param vX1 The x coordinate where the glyph image is rendered.
    @param vY1 The y coordinate where the glyph image is rendered.
    @param kGlyphImage The glyph image containing a 8 bit alpha image of the glyph.
    */
    virtual void RenderGlyphToScreen(const eC_Value& vX1, const eC_Value& vY1, CGUIImageData& kGlyphImage);

    /** Copy-constructor.
    @param kSource Source object to be copied.
    */
    CFntWrapFreeTypeDAVE(const CFntWrapFreeTypeDAVE& kSource);

    /** Operator= method.
    @param kSource Source object to be copied.
    @return This instance.
    */
    CFntWrapFreeTypeDAVE& operator=(const CFntWrapFreeTypeDAVE& kSource);

    eC_UInt m_uiGlyphCacheInitialSize;
    eC_Bool m_bGlyphCacheAutoResize;
    eC_UInt m_uiGlyphCacheResizeBy;
};
#endif
