/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GFX_ENV_DAVE__H__
#define __GFX_ENV_DAVE__H__

#include "GfxEnv.h"

#include "dave_base.h"

/**
This class implements the D/AVE-emulation of a display-controller
it uses the d1-functions of D/AVE to access defined layers

it supports:
- position
- size
- alpha-blend
*/
class CGfxEnvDAVE : public CGfxEnv
{
public:
    /** Create Instance
    @param eSurfaceMode surface-mode
    @param eScreenFormat screen-format
    @param uiWidth width
    @param uiHeight height
    @param uiBufferCount number of buffers to use
    @param bResizable true if resizable
    @return true if successful
    */
    static eC_Bool CreateInstance(
        const SurfaceMode_t& eSurfaceMode,
        const CGUICommonEnums::ImageType_t& eScreenFormat,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& uiBufferCount,
        const eC_Bool& bResizable);

    /** Initialize display sub-system
    @return returns 1 on success
    */
    virtual eC_UInt InitDisplay();

    /** De-Initialize the display sub-system
    */
    virtual void DeInitDisplay();

    /** Initialize layer-configuration
    @param kLayerConfig layer-configuration
    */
    virtual void InitLayer(const LayerConfiguration_t& kLayerConfig);

    /** Update layer
    @param eLayerConfig layer-configuration
    @param eUpdateReason reason for update
    */
    virtual void UpdateLayer(const LayerConfiguration_t& eLayerConfig, const LayerUpdate_t& eUpdateReason);

    /** Refresh the layer
    @param uiLayerID id of layer
    @param pkRenderBuffer address of render-buffer
    @param pkDisplayBuffer address of display-buffer
    @param uiWidth width of layer
    @param uiHeight height of layer
    @return true if successful
    */
    virtual eC_Bool UpdateLayerBuffers(const eC_UInt& uiLayerID, void* pkRenderBuffer, void* pkDisplayBuffer, const eC_UInt& uiWidth, const eC_UInt& uiHeight);

    virtual void StartOfRefresh();

    /** Gets executed at the end of the refresh
    */
    virtual void EndOfRefresh();

    /** Set the device for the wrapper
    @param pkDevice platform-specific device
    */
    virtual void SetDevice(void* pkDevice);

    /** Return the merge-buffer
    @return buffer
    */
    virtual void* GetMergeBuffer() const;

protected:
    /** Constructor
    @param eSurfaceMode OpenGL or Framebuffer
    @param eScreenFormat screen-format
    @param uiWidth width
    @param uiHeight height
    @param uiBufferCount number of buffers to use
    @param bResizable true if resizable
    @param bUseDisplayEmulation true if dc-emulation should be used
    */
    CGfxEnvDAVE(
        const SurfaceMode_t& eSurfaceMode,
        const CGUICommonEnums::ImageType_t& eScreenFormat,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& uiBufferCount = 1,
        const eC_Bool& bResizable = false,
        const eC_Bool& bUseDisplayEmulation = false);
    virtual ~CGfxEnvDAVE();

private:
    eC_UInt GetD1PixelFormat(const CGUICommonEnums::ImageType_t& ePixFormat) const;

private:
    d1_device* m_dc_handle;
    eC_Bool m_bUseDisplayEmulation;
};

#endif
