/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GFXWRAPDAVE__H_
#define GFXWRAPDAVE__H_

#include "GfxWrap.h"

#include "DAVETypes.h"
#include "DAVEScreen.h"

#include "GUIImageLoader.h"
#include "GUIImageDataDAVE.h"

#include "GUILayerConfig.h"

// Dave driver
#include "dave_base.h"
#include "dave_driver.h"

// If BSP supports TES own Device driver, then use it


/// D/AVE implementation of the graphics wrapper.

/**
This is the D/AVE implementation of the graphics wrapper.
For details see description of its base class CGfxWrap and the D/AVE documentation.

Since data needs to be available for the D/AVE-driver until the current frame is finished,
the wrapper supports working memory which will be allocated at the beginning and re-used
during runtime. The size of the working buffer can be specified when creating the wrapper.

See @ref subsec_limitations_GfxWrapDAVE "CGfxWrapDAVE Platform Specific Limitations"
*/

class CGfxWrapDAVE : public CGfxWrap
{
public:
    /** Create instance
    @param pkGfxEnv GfxEnv
    @param uiWorkingMemorySize size of working memory (default: 64kB)
    @return true if successful
    */
    static eC_Bool CreateInstance(
        CGfxEnv* pkGfxEnv,
        const eC_UInt& uiWorkingMemorySize = 64000);

    /** Create a screen according to data from dc-wrap
    */
    virtual void CreateScreen();

    virtual void DestroyScreen();

    virtual eC_Bool Refresh(const RefreshCall_t eRefreshCallType);

    virtual void SetForegroundColorImpl(const eC_UByte &ubRed, const eC_UByte &ubGreen, const eC_UByte &ubBlue, const eC_UByte &ubAlpha = 255);
    virtual eC_Value SetLineWidth(const eC_Value& vWidth);
    virtual void Line(const eC_Value& vX1, const eC_Value& vY1, const eC_Value& vX2, const eC_Value& vY2);
    virtual void Rect(const eC_Value& vX1, const eC_Value& vY1, const eC_Value& vX2, const eC_Value& vY2);
    virtual void FilledRect(const eC_Value& vX1, const eC_Value& vY1, const eC_Value& vX2, const eC_Value& vY2);

    virtual void Ellipse(
        const eC_Value& vX1, const eC_Value& vY1,
        const eC_Value& vX2, const eC_Value& vY2,
        const eC_Bool &bFilled = true);

    virtual void Arc(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Value& vStartAngle,
        const eC_Value& vEndAngle,
        const eC_Value& vRotationAngle,
        const eC_Bool &bFilled);

    virtual void Ring(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Value& vRingWidth,
        const eC_Value& vStartAngle,
        const eC_Value& vEndAngle,
        const eC_Value& vRotationAngle,
        const eC_Bool & bFilled);

    virtual void Polygon(
        eC_Value* pvPoints,
        const eC_UInt& uiCount,
        const eC_Bool& bFilled);

    virtual void SetCliprect(const CGUIRect &NewClipRect);

    /** Implementation of image blitting.
        For detailed explanations, please refer to CGfxWrap::BlitImgExtImpl
        PLEASE NOTE that rotated blits are not supported for DAVE!
        @param eID          Identifier of the image to be blitted.
        @param uiSrcX       X coordinate of upper left corner of the relevant area within the source image
        @param uiSrcY       Y coordinate of upper left corner of the relevant area within the source image
        @param uiSrcWidth   Width of the of the relevant area within the source image
        @param uiSrcHeight  Height of the of the relevant area within the source image
        @param vDestX       Destination X position in absolute coordinates
        @param vDestY       Destination Y position in absolute coordinates
        @param vDestWidth   Destination width
        @param vDestHeight  Destination height
        @param ubAlpha      Degree of transparency. 0=transparent 255=opaque  Default is opaque.
        @param vAngle       Angle in degrees by which the image will be rotated
        @param vRotCenterX  Rotation center X. In range of 0 to 1  (like U/V coordinates)
        @param vRotCenterY  Rotation center Y. In range of 0 to 1  (like U/V coordinates)
    */
    virtual void BlitImgExtImpl(
        const ImageResource_t &eID,
        const eC_UInt &uiSrcX, const eC_UInt &uiSrcY, const eC_UInt &uiSrcWidth, const eC_UInt &uiSrcHeight,
        const eC_Value& vDestX, const eC_Value& vDestY, const eC_Value& vDestWidth, const eC_Value& vDestHeight,
        const eC_UByte &ubAlpha = 255,
        const eC_Value& vAngle = eC_FromFloat(0),
        const eC_Value& vRotCenterX = eC_FromFloat(0.5), const eC_Value& vRotCenterY = eC_FromFloat(0.5));

    virtual void BlitImgExtImpl(const CGUIImageData& kImageData, const eC_Value& vAbsDestX, const eC_Value& vAbsDestY);

    virtual eC_UInt GetImgWidthImpl(const ImageResource_t &eID) const;
    virtual eC_UInt GetImgHeightImpl(const ImageResource_t &eID) const;
    virtual eC_UInt GetImageSize(const ImageResource_t& eID) const {return 0;}

    virtual void RefreshImage(const ImageResource_t& eID, void* pkMemory);

    /** Set new alpha-value
    @param ubAlpha The new alpha value
    */
    void SetAlpha(eC_UByte ubAlpha);

    /// Checks DAVE device for errors and outputs a log-message with error-code and description.
    void CheckDAVEError();

    virtual eC_Bool DrawToBitmap(GUIBitmapPlaneID_t uiBitmapID);
    virtual eC_Bool DrawToScreen();

    virtual void SetTargetLayer(const eC_UInt& uiLayerID);

    /** WARNING!!!! when using RA6 inline functions return trash and can't be used */
    /** Returns a pointer to the internal dave device handle.
    @return The dave device handle.
    */
    d2_device* GetDaveDevice();

    virtual eC_UInt GetVirtualScreenWidth();
    virtual eC_UInt GetVirtualScreenHeight();

    virtual eC_UInt GetPhysicalScreenWidth();
    virtual eC_UInt GetPhysicalScreenHeight();

    /** Get D1-pixelformat
    @param ePixFormat pixel-format
    @return D1-pixelformat
    */
    static eC_UInt GetD1PixelFormat(const CGUICommonEnums::ImageType_t& ePixFormat);

    /** Get D2-pixelformat
    @param ePixFormat pixel-format
    @return D2-pixelformat
    */
    static eC_UInt GetD2PixelFormat(const CGUICommonEnums::ImageType_t& ePixFormat);

    virtual eC_Bool SupportCompressedFonts() const { return true; }

    virtual void LoadGlyphData(
        CGUIImageData* pkImageData,
        const eC_UInt& uiGlyphWidth,
        const eC_UInt& uiGlyphHeight,
        eC_UByte* pkGlyphBitmap);

    virtual void UnloadGlyphData(CGUIImageData* pkImageData);

protected:
    /** Initialize screens.
    */
    void InitScreens();

    /** Start a new render-buffer
    */
    void StartRenderBuffer();

    /** end the current render-buffer
    */
    void EndRenderBuffer();

    virtual void LoadImgImpl(const eC_String& kPath, const ImageResource_t &eID);
    virtual void LoadImgImpl(
        void* pkMemory,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& eImageType,
        const ImageResource_t &eID);

    /** Load image in RAW-format
    @param pkImageFile file to load
    @param kImageData filled imagedata
    @return true on success
    */
    virtual eC_Bool LoadImgRAW(eC_File* pkImageFile, CGUIImageDataDAVE& kImageData);

    /** Load image in RLE-format
    @param pkImageFile file to load
    @param kImageData filled imagedata
    @return true on success
    */
    virtual eC_Bool LoadImgRLE(eC_File* pkImageFile, CGUIImageDataDAVE& kImageData);

    /** Load image using decoder if present
    @param kPath file to load
    @param uiImageIndex index for image
    @return true on success
    */
    virtual eC_Bool LoadImgDecoder(const eC_String& kPath, const eC_UInt& uiImageIndex);

    virtual void UnloadImg(const ImageResource_t &eID);

    virtual eC_Bool ImageExists(const ImageResource_t eID) const;

    virtual CGUIBitmapPlane* CreateGfxDependentBitmap(
        GUIBitmapPlaneID_t uiNewBitmapID,
        const CGUIObject& rkParentObject,
        const eC_Value& vWidth,
        const eC_Value& vHeight);

    /** Init the graphics wrapper
    @return true on success
    */
    eC_Bool Init(void);

    /** De-init the graphics wrapper
    @return true on success
    */
    eC_Bool DeInit(void);

    /** Increases the size of the array of image resources
    @param uiNOFImages number of images
    @return true on success
    */
    eC_Bool SetNOFImagesImpl(const eC_UInt uiNOFImages);

    /** Constructor. Please refer to the DAVE API documentation for details regarding dave-display modes.
    @param pkGfxEnv
    @param uiWorkingMemorySize
    */
    CGfxWrapDAVE(CGfxEnv* pkGfxEnv, const eC_UInt& uiWorkingMemorySize);

    /// Destructor
    virtual ~CGfxWrapDAVE(void);

protected:
    /// config for screen
    typedef struct
    {
        eC_UInt uiID; ///< id of screen
        CDAVEScreen* pScreen; ///< pointer to screens
    } ScreenConfig_t;

protected:
    /// Pointer to memory areas of configured framebuffers
    ScreenConfig_t* m_pkScreens;

    /// number of screen
    eC_UInt m_uiNumberOfScreens;

    /// Dave L1 Mode
    eC_Int  m_iD1Mode;
    /// Dave L2 Mode
    eC_Int  m_iD2Mode;
    /// a device for DAVE
    d2_device* m_pkD2Handle;
    /// A handle for the display controller
    d1_device* m_pkD1Handle;

    /// a dave context
    d2_context *m_pkD2Context;
    /// the array of the loaded images
    eC_TArray<CGUIImageDataDAVE*> m_pkImages;

    /// true if memory is seperated (e.g. Linux)
    eC_Bool m_bSeperatedMemory;

    /// A handle for the display controller
    dc_handle m_dc_handle;

    /// if frame has started
    eC_Bool m_bFrameStarted;
    /// a single renderbuffer for immediate mode
    d2_renderbuffer* m_pkRenderBuffer;
};
#endif
