/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FNTWRAP_GL_FONTATLAS__H__
#define __FNTWRAP_GL_FONTATLAS__H__

//FreeType includes
#include "ft2build.h"
#include FT_GLYPH_H

#include "eC_Types.h"
#include "eC_TArray.h"
#include "GUIFontResource.h"

/** This class is used by FntWrapFreeTypeGL. It stores the texture-atlas information for one Font.
*/
class CFntWrapGLFontAtlas
{
public:
    /// Struct representing one character inside the cache array
    struct FontAtlasRecord_t
    {
        FontAtlasRecord_t() :
            m_uiUnicode(0xFFFF),
            m_iGlyphXPos(0),
            m_iGlyphYPos(0),
            m_iGlyphWidth(0),
            m_iGlyphHeight(0),
            m_uiXAdvance(0),
            m_uiXPos(0),
            m_uiYPos(0)
        {}

        eC_UInt m_uiUnicode; ///< codepoint for glyph
        eC_Int m_iGlyphXPos; ///< x-position of glyph in atlas
        eC_Int m_iGlyphYPos; ///< y-position of glyph in atlas
        eC_Int m_iGlyphWidth; ///< width of glyph
        eC_Int m_iGlyphHeight; ///< height of glyph
        eC_UInt m_uiXAdvance; ///< x-advance of glyph
        eC_UInt m_uiXPos; ///< x-corretion of glyph
        eC_UInt m_uiYPos; ///< y-correction of lgyph
    };

    /** Constructor
    @param reFontID ID of selected font.
    @param uiNumberOfRows Initial number of rows for the texture atlas.
    */
    CFntWrapGLFontAtlas(
        const FontResource_t& reFontID,
        const eC_UInt& uiNumberOfRows);

    /** Destructor */
    virtual ~CFntWrapGLFontAtlas();

    /** This method returns a struct with the wanted character's information.
    It adds a character to the cache array if it does not exist.
    @param uiUnicode Unicode of wanted character.
    @return Struct of wanted character.
    */
    FontAtlasRecord_t GetChar(const eC_UInt &uiUnicode);

    /** Returns the height of current font.
    @return Font's height.
    */
    inline eC_Int GetHeight() const { return m_iFontHeight; }

    /** Returns the ascender of the current font.
    @return Font's ascender.
    */
    inline eC_Int GetAscender() const { return m_iAscender; }

    /** Returns the descender of the current font.
    @return Font's descender.
    */
    inline eC_Int GetDescender() const { return m_iDescender; }

    /** Returns the internal leading of the current font.
    @return Font's internal leading.
    */
    inline eC_Int GetInternalLeading() const { return m_iInternalLeading; }

    /** Returns the internal ID for the used texture
    @return texture-id
    */
    eC_UInt GetTextureID() const { return m_uiTextureID; }

    /** Return the width of the used texture
    @return width of texture
    */
    eC_UInt GetTextureWidth() const { return m_uiTextureWidth; }

    /** Return the height of the used texture
    @return height of texture
    */
    eC_UInt GetTextureHeight() const { return m_uiTextureHeight; }

    /** return the current face
    @return face
    */
    virtual void* GetFace() = 0;

protected:
    /** Adds a character to the cache array and sorts array afterwards.
    @param uiUnicode Unicode to add to array.
    @return Struct of just added character.
    */
    virtual FontAtlasRecord_t AddGlyph(const eC_UInt &uiUnicode) = 0;

    /** create a texture with the given dimensions
    @param iTextureWidth width
    @param iTextureHeight height
    @param uiLastTextureID ID of last generated texture for font
    @return id of the new texture
    */
    eC_UInt CreateTexture(
        const eC_Int& iTextureWidth,
        const eC_Int& iTextureHeight,
        const eC_UInt& uiLastTextureID = 0);

    /** delete the texture
    @param uiTextureID
    */
    void DeleteTexture(const eC_UInt& uiTextureID);

    /** add the information for a new glyph
    @param uiUnicode code-point
    @param uiWidth width of glyph
    @param iGlyphXPos x-position
    @param iGlyphYPos y-position
    @param iGlyphWidth width
    @param iGlyphHeight height
    @param uiXPos x-position inside texture
    @param uiYPos y-position inside texture
    @return index of the new glyph
    */
    eC_UInt InsertGlyphInformation(
        const eC_UInt& uiUnicode,
        const eC_UInt& uiWidth,
        const eC_Int& iGlyphXPos,
        const eC_Int& iGlyphYPos,
        const eC_Int& iGlyphWidth,
        const eC_Int& iGlyphHeight,
        const eC_UInt& uiXPos,
        const eC_UInt& uiYPos);

private:
    /** Searches for a character inside the cache array by using divide and
    conquer.
    @param uiUnicode Character to search for.
    @return Position in array of -1 if not found.
    */
    eC_Int FindChar(const eC_UInt &uiUnicode);

    /** Swaps two given values.
    @param pPosA Pointer of the first value.
    @param pPosB Pointer of the second value.
    */
    void Swap(
        FontAtlasRecord_t *pPosA,
        FontAtlasRecord_t *pPosB);

    /** Sorts the cache list by unicode in ascending order using bubble sort
    algorithm.*/
    void Sort();

    /** Copy-constructor.
    Dummy implementation to avoid unintended use of compiler-generated default    */
    CFntWrapGLFontAtlas(const CFntWrapGLFontAtlas& kSource);

    /** Operator= method.
    Dummy implementation to avoid unintended use of compiler-generated default    */
    CFntWrapGLFontAtlas& operator=(const CFntWrapGLFontAtlas& kSource);

protected:
    FontResource_t m_eFontID;           ///< id of loaded font
    eC_Int m_iAscender;                 ///< ascender
    eC_Int m_iDescender;                ///< descender
    eC_Int m_iInternalLeading;          ///< internal leading
    eC_Int m_iFontWidth;                ///< width of font
    eC_Int m_iFontHeight;               ///< height of font

    eC_UInt m_uiTotalNumberOfRows;      ///< number of rows which can be used for storage
    eC_UInt m_uiCurrentFillRow;         ///< current row for new characters
    eC_UInt m_uiRemainingSpace;         ///< remaining pixels of storage for current row
    eC_UInt m_uiTextureID;              ///< texture-id for the atlas
    eC_UInt m_uiTextureWidth;           ///< width of generated texture
    eC_UInt m_uiTextureHeight;          ///< height of generated texture

    eC_TArray<FontAtlasRecord_t> m_asGlyphCache;  ///< The dynamic cache array.
    eC_UInt m_uiNumberOfCachedGlyphs;   ///< number of cached glyphs
    eC_Bool m_bArrayNeedsSorting;       ///< Flag indicating that the internal array of stored characters needs to be sorted
};
#endif
