/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GFXWRAPGL__H_
#define GFXWRAPGL__H_

#include "GfxWrap.h"

#include "GL_API_Mappings.h"

#include "GUIImageDataGL.h"
#include "GUIConfigDebug.h"

#ifdef GUILIANI_GFXOGLDEBUG
/// Helper macro for convenient checks for OpenGL errors in debug builds.
#define CHECK_GL_ERROR(a)   CGfxWrapGL::CheckGLError(__FUNCTION__, a);
#else
#define CHECK_GL_ERROR(a)
#endif

/// class to load and use an OpenGL-shader
class CGfxWrapGLShader
{
public:
    /// Constructor
    CGfxWrapGLShader();

    /// Destructor
    ~CGfxWrapGLShader();

    /** Load a shader
    @param cVertexShader
    @param cFragmentShader
    @return true if successfully loaded
    */
    eC_Bool Load(const char* cVertexShader, const char* cFragmentShader);

    /** Return shader-id
    @return id
    */
    eC_UInt GetShaderID() const { return m_uiShaderProgramID; }

    /** Use shader */
    void Activate() const;

private:
    eC_UInt m_uiVertexShaderHandle;
    eC_UInt m_uiFragmentShaderHandle;
    eC_UInt m_uiShaderProgramID;
};

/// OpenGL graphic wrapper base class. This is the base class for all OpenGL related graphic wrappers (OpenGL and OpenGL-ES).
/** To understand how Guiliani is handling coordinatesystems under OpenGL please @see cvPixelCenterOffset.*/
class CGfxWrapGL : public CGfxWrap
{
    friend class CGUIBitmapPlane_GL; ///< @todo to access private functions

public:
    /// Offset to get from OpenGL's coordinates to the pixel center.
    /// Note: OpenGL Pixel Coordinates do not refer to the "center" of the pixel, but to the edges between them.
    /// This means 0/0 would not point to the center of the top/left pixel, but instead to the top/left corner of this pixel
    /// Therefore, we move the coordinates to the center of the pixel by adding 0.375 pixels because of better compatibility
    /// to more graphic cards.
    static const  eC_Value cvPixelCenterOffset;

    /** create screen
    @param uiWidth
    @param uiHeight
    */
    void CreateScreen(const eC_UInt &uiWidth, const eC_UInt &uiHeight);

    virtual eC_UInt GetVirtualScreenWidth();
    virtual eC_UInt GetPhysicalScreenWidth();
    virtual eC_UInt GetVirtualScreenHeight();
    virtual eC_UInt GetPhysicalScreenHeight();
    virtual void SetScreenSize(const eC_UInt &uiWidth, const eC_UInt &uiHeight);

    virtual void StoreWrapperState();
    virtual void RestoreWrapperState();

    /** Sets the foreground color via r, g and b values and an alpha value
    @param ubRed Red intensity within a range of 0 - 255
    @param ubGreen Green intensity within a range of 0 - 255
    @param ubBlue Blue intensity within a range of 0 - 255
    @param ubAlpha alpha value
    */
    virtual void SetForegroundColorImpl(
        const eC_UByte &ubRed,
        const eC_UByte &ubGreen,
        const eC_UByte &ubBlue,
        const eC_UByte& ubAlpha = 255);

    virtual eC_Value SetLineWidth(const eC_Value& vWidth);

    /** Draws a line from (vX1, vY1) to (vX2, vY2)
    @param vX1 X-Coordinate of starting point
    @param vY1 Y-Coordinate of starting point
    @param vX2 X-Coordinate of ending point
    @param vY2 Y-Coordinate of ending point
    */
    virtual void Line(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2);

    /** Draws a rectangle with the top left edge at (vX1, vY1) and bottom right edge at (vX2, vY2)
    @param vX1 X-Coordinate of top left corner
    @param vY1 Y-Coordinate of top left corner
    @param vX2 X-Coordinate of bottom right corner
    @param vY2 Y-Coordinate of bottom right corner
    */
    virtual void Rect(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2);

    /** Draws a filled rectangle with the top left edge at (vX1, vY1) and bottom right edge at (vX2, vY2)
    @param vX1 X-Coordinate of top left corner
    @param vY1 Y-Coordinate of top left corner
    @param vX2 X-Coordinate of bottom right corner
    @param vY2 Y-Coordinate of bottom right corner
    */
    virtual void FilledRect(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2);

    /** Draws an ellipse which is complete inside the rectangle described through
    the rectangle the top left edge at (vX1, vY1) and bottom right edge at (vX2, vY2).
    The ellipse will be filled out if the last parameter is true
    @param vX1 X-Coordinate of bounding rectangle's top left corner
    @param vY1 Y-Coordinate of bounding rectangle's top left corner
    @param vX2 X-Coordinate of bounding rectangle's bottom right corner
    @param vY2 Y-Coordinate of bounding rectangle's bottom right corner
    @param bFilled Specifies whether the ellipse will be drawn outlined (false) or filled (true)
    */
    virtual void Ellipse(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Bool &bFilled);

    /** Draws an arc which is completly inside the rectangle described
        through the rectangle the top left edge at (vAbsX1, vAbsY1)
        and bottom right edge at (vAbsX2, vAbsY2).
        The arc will be filled out if the last parameter is true
    @param vX1 X-coordinate of bounding rectangle's top left corner
    @param vY1 Y-Coordinate of bounding rectangle's top left corner
    @param vX2 X-coordinate of bounding rectangle's bottom right corner
    @param vY2 Y-Coordinate of bounding rectangle's bottom right corner
    @param vStartAngle start-angle
    @param vEndAngle end-angle
    @param vRotationAngle rotation-angle
    @param bFilled Specifies whether the arc will be drawn outlined (false) or filled (true)
    */
    virtual void Arc(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Value& vStartAngle,
        const eC_Value& vEndAngle,
        const eC_Value& vRotationAngle,
        const eC_Bool &bFilled);

    virtual void Ring(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Value& vRingWidth,
        const eC_Value& vStartAngle,
        const eC_Value& vEndAngle,
        const eC_Value& vRotationAngle,
        const eC_Bool & bFilled);

    virtual void Polygon(
        eC_Value* pvPoints,
        const eC_UInt& uiCount,
        const eC_Bool& bFilled);

    // ---------------------------------------------------------------
    // Clipping and invalidated rectangles
    // ---------------------------------------------------------------

    /** Sets a clipping rectangle.
    All drawing operations will be limited to the region within the clipping rectangle. This is very
    useful for advanded features such as soft-scrolling. Note that the clipping rectangle is not
    allowed to exceed the invalidated rectangle, since this would lead to visual artifacts.
    @param rkNewClipRect A normalised rectangle that includes all pixels that could be drawn
    by the functions of the graphics wrapper. The rectangle includes the
    border. All pixels outside will be left untouched.
    */
    virtual void SetCliprect(const CGUIRect &rkNewClipRect);

    virtual eC_Bool ImageExists(const ImageResource_t eID) const;

    /** Gets pointer to an image data object.
    @param eImageID Image data object to be returned.
    @return Pointer to image data object.
    */
    CGUIImageDataGL* GetImageData(const ImageResource_t &eImageID);

    /** Sets the default 2D view used by the gfx wrapper for 2D drawing operations.
    This method uses the dimensions returned by GetVirtualScreenWidth() and GetVirtualScreenHeight().
    */
    void Set2DView();

    /** Sets the default 2D view used by the gfx wrapper for 2D drawing operations.
    @param uiWidth Width of Viewport
    @param uiHeight Height of Viewport
    */
    void Set2DView(const eC_UInt uiWidth, const eC_UInt uiHeight);

    /** Checks for occurency of OpenGL Erros.
    All OpenGL errors (error code and text) since the last call of this function (or glGetError) are log in the Guiliani Log.
    @param kCallingMethod String which will be added to the Log entry.
    @param kFunction name of function
    @return True if successful, False otherwise
    */
    static eC_Bool CheckGLError(const eC_String& kCallingMethod, const eC_String& kFunction = "");

    virtual eC_Bool DrawToBitmap(GUIBitmapPlaneID_t uiBitmapID);

    virtual eC_Bool DrawToScreen();

    /** Returns the TextureID and generates the texture if needed
    @param eImageID The image ID
    @return The texture ID
    */
    const eC_UInt GetTextureID(const ImageResource_t &eImageID);

    /**
    * Applies a perspective projection matrix to current GL_PROJECTION. Old
    * projection matrix will be overwritten. Projection matrices are used
    * to project 3D space coordinates to a 2D screen.
    *
    * OpenGL and OpenGL-ES provides projection matrix creation for parallel
    * projection mode (glOrtho), but for perspective projection invokation
    * of the GLU library is needed. To avoid this dependency, this
    * implementation provides this.
    *
    * The matrix is calculated the following way:
    *
    *     (1/w) (0) (      0      ) (      0      )
    * M = (0) (1/h) (      0      ) (      0      )
    *     (0) (0) (-(near+far)/d) (-2*near*far/d)
    *     (0) (0) (     -1      ) (      0      )
    *
    * where:
    *
    * h = tan(fovy/2)
    * w = aspectRatio * h
    * d = far - near
    *
    * @param vFovy Field of view in the vertical direction in DEG [0; 90].
    * @param vAspect Aspect ratio of the viewport, which is width divided by
    *  height.
    * @param vNear Distance to front clipping plane.
    * @param vFar Distance to back clipping plane.
    */
    void SetPerspectiveProjection(
        const eC_Value& vFovy,
        const eC_Value& vAspect,
        const eC_Value& vNear,
        const eC_Value& vFar);

    /** Helper for initialization
    @return
    */
    virtual eC_Bool InitView();

    /** Returns the currently used vertex buffer object.
    @return Vertex buffer object.
    */
    inline eC_UInt GetVBO() const
    {
        return m_uiVertexVBO;
    }

#if defined (GUILIANI_USE_OGL) && !defined (SYSTEM_NAME_DARWIN)
    /** Function pointers for Vertex Buffer Objects. */
    static PFNGLBINDBUFFERPROC              glBindBuffer_Guiliani;
    static PFNGLBUFFERDATAPROC              glBufferData_Guiliani;
    static PFNGLGENBUFFERSPROC              glGenBuffers_Guiliani;
    static PFNGLDELETEBUFFERSPROC           glDeleteBuffers_Guiliani;

    /** Function pointers for Frame Buffer Objects. */
    static PFNGLGENFRAMEBUFFERSPROC         glGenFramebuffers_Guiliani;
    static PFNGLBINDFRAMEBUFFERPROC         glBindFramebuffer_Guiliani;
    static PFNGLFRAMEBUFFERTEXTURE2DPROC    glFramebufferTexture2D_Guiliani;
    static PFNGLCHECKFRAMEBUFFERSTATUSPROC  glCheckFramebufferStatus_Guiliani;
    static PFNGLDELETEFRAMEBUFFERSPROC      glDeleteFramebuffers_Guiliani;

#if GUILIANI_USE_GL_SHADERS == 1
    /** Function pointers for Shaders (OpenGL 2.0) */
    static PFNGLBLENDEQUATIONSEPARATEPROC   glBlendEquationSeparate_Guiliani;
    static PFNGLDRAWBUFFERSPROC             glDrawBuffers_Guiliani;
    static PFNGLSTENCILOPSEPARATEPROC       glStencilOpSeparate_Guiliani;
    static PFNGLSTENCILFUNCSEPARATEPROC     glStencilFuncSeparate_Guiliani;
    static PFNGLSTENCILMASKSEPARATEPROC     glStencilMaskSeparate_Guiliani;
    static PFNGLATTACHSHADERPROC            glAttachShader_Guiliani;
    static PFNGLBINDATTRIBLOCATIONPROC      glBindAttribLocation_Guiliani;
    static PFNGLCOMPILESHADERPROC           glCompileShader_Guiliani;
    static PFNGLCREATEPROGRAMPROC           glCreateProgram_Guiliani;
    static PFNGLCREATESHADERPROC            glCreateShader_Guiliani;
    static PFNGLDELETEPROGRAMPROC           glDeleteProgram_Guiliani;
    static PFNGLDELETESHADERPROC            glDeleteShader_Guiliani;
    static PFNGLDETACHSHADERPROC            glDetachShader_Guiliani;
    static PFNGLDISABLEVERTEXATTRIBARRAYPROC glDisableVertexAttribArray_Guiliani;
    static PFNGLENABLEVERTEXATTRIBARRAYPROC glEnableVertexAttribArray_Guiliani;
    static PFNGLGETACTIVEATTRIBPROC         glGetActiveAttrib_Guiliani;
    static PFNGLGETACTIVEUNIFORMPROC        glGetActiveUniform_Guiliani;
    static PFNGLGETATTACHEDSHADERSPROC      glGetAttachedShaders_Guiliani;
    static PFNGLGETATTRIBLOCATIONPROC       glGetAttribLocation_Guiliani;
    static PFNGLGETPROGRAMIVPROC            glGetProgramiv_Guiliani;
    static PFNGLGETPROGRAMINFOLOGPROC       glGetProgramInfoLog_Guiliani;
    static PFNGLGETSHADERIVPROC             glGetShaderiv_Guiliani;
    static PFNGLGETSHADERINFOLOGPROC        glGetShaderInfoLog_Guiliani;
    static PFNGLGETSHADERSOURCEPROC         glGetShaderSource_Guiliani;
    static PFNGLGETUNIFORMLOCATIONPROC      glGetUniformLocation_Guiliani;
    static PFNGLGETUNIFORMFVPROC            glGetUniformfv_Guiliani;
    static PFNGLGETUNIFORMIVPROC            glGetUniformiv_Guiliani;
    static PFNGLGETVERTEXATTRIBDVPROC       glGetVertexAttribdv_Guiliani;
    static PFNGLGETVERTEXATTRIBFVPROC       glGetVertexAttribfv_Guiliani;
    static PFNGLGETVERTEXATTRIBIVPROC       glGetVertexAttribiv_Guiliani;
    static PFNGLGETVERTEXATTRIBPOINTERVPROC glGetVertexAttribPointerv_Guiliani;
    static PFNGLISPROGRAMPROC               glIsProgram_Guiliani;
    static PFNGLISSHADERPROC                glIsShader_Guiliani;
    static PFNGLLINKPROGRAMPROC             glLinkProgram_Guiliani;
    static PFNGLSHADERSOURCEPROC            glShaderSource_Guiliani;
    static PFNGLUSEPROGRAMPROC              glUseProgram_Guiliani;
    static PFNGLUNIFORM1FPROC               glUniform1f_Guiliani;
    static PFNGLUNIFORM2FPROC               glUniform2f_Guiliani;
    static PFNGLUNIFORM3FPROC               glUniform3f_Guiliani;
    static PFNGLUNIFORM4FPROC               glUniform4f_Guiliani;
    static PFNGLUNIFORM1IPROC               glUniform1i_Guiliani;
    static PFNGLUNIFORM2IPROC               glUniform2i_Guiliani;
    static PFNGLUNIFORM3IPROC               glUniform3i_Guiliani;
    static PFNGLUNIFORM4IPROC               glUniform4i_Guiliani;
    static PFNGLUNIFORM1FVPROC              glUniform1fv_Guiliani;
    static PFNGLUNIFORM2FVPROC              glUniform2fv_Guiliani;
    static PFNGLUNIFORM3FVPROC              glUniform3fv_Guiliani;
    static PFNGLUNIFORM4FVPROC              glUniform4fv_Guiliani;
    static PFNGLUNIFORM1IVPROC              glUniform1iv_Guiliani;
    static PFNGLUNIFORM2IVPROC              glUniform2iv_Guiliani;
    static PFNGLUNIFORM3IVPROC              glUniform3iv_Guiliani;
    static PFNGLUNIFORM4IVPROC              glUniform4iv_Guiliani;
    static PFNGLUNIFORMMATRIX2FVPROC        glUniformMatrix2fv_Guiliani;
    static PFNGLUNIFORMMATRIX3FVPROC        glUniformMatrix3fv_Guiliani;
    static PFNGLUNIFORMMATRIX4FVPROC        glUniformMatrix4fv_Guiliani;
    static PFNGLVALIDATEPROGRAMPROC         glValidateProgram_Guiliani;
    static PFNGLVERTEXATTRIB1DPROC          glVertexAttrib1d_Guiliani;
    static PFNGLVERTEXATTRIB1DVPROC         glVertexAttrib1dv_Guiliani;
    static PFNGLVERTEXATTRIB1FPROC          glVertexAttrib1f_Guiliani;
    static PFNGLVERTEXATTRIB1FVPROC         glVertexAttrib1fv_Guiliani;
    static PFNGLVERTEXATTRIB1SPROC          glVertexAttrib1s_Guiliani;
    static PFNGLVERTEXATTRIB1SVPROC         glVertexAttrib1sv_Guiliani;
    static PFNGLVERTEXATTRIB2DPROC          glVertexAttrib2d_Guiliani;
    static PFNGLVERTEXATTRIB2DVPROC         glVertexAttrib2dv_Guiliani;
    static PFNGLVERTEXATTRIB2FPROC          glVertexAttrib2f_Guiliani;
    static PFNGLVERTEXATTRIB2FVPROC         glVertexAttrib2fv_Guiliani;
    static PFNGLVERTEXATTRIB2SPROC          glVertexAttrib2s_Guiliani;
    static PFNGLVERTEXATTRIB2SVPROC         glVertexAttrib2sv_Guiliani;
    static PFNGLVERTEXATTRIB3DPROC          glVertexAttrib3d_Guiliani;
    static PFNGLVERTEXATTRIB3DVPROC         glVertexAttrib3dv_Guiliani;
    static PFNGLVERTEXATTRIB3FPROC          glVertexAttrib3f_Guiliani;
    static PFNGLVERTEXATTRIB3FVPROC         glVertexAttrib3fv_Guiliani;
    static PFNGLVERTEXATTRIB3SPROC          glVertexAttrib3s_Guiliani;
    static PFNGLVERTEXATTRIB3SVPROC         glVertexAttrib3sv_Guiliani;
    static PFNGLVERTEXATTRIB4NBVPROC        glVertexAttrib4Nbv_Guiliani;
    static PFNGLVERTEXATTRIB4NIVPROC        glVertexAttrib4Niv_Guiliani;
    static PFNGLVERTEXATTRIB4NSVPROC        glVertexAttrib4Nsv_Guiliani;
    static PFNGLVERTEXATTRIB4NUBPROC        glVertexAttrib4Nub_Guiliani;
    static PFNGLVERTEXATTRIB4NUBVPROC       glVertexAttrib4Nubv_Guiliani;
    static PFNGLVERTEXATTRIB4NUIVPROC       glVertexAttrib4Nuiv_Guiliani;
    static PFNGLVERTEXATTRIB4NUSVPROC       glVertexAttrib4Nusv_Guiliani;
    static PFNGLVERTEXATTRIB4BVPROC         glVertexAttrib4bv_Guiliani;
    static PFNGLVERTEXATTRIB4DPROC          glVertexAttrib4d_Guiliani;
    static PFNGLVERTEXATTRIB4DVPROC         glVertexAttrib4dv_Guiliani;
    static PFNGLVERTEXATTRIB4FPROC          glVertexAttrib4f_Guiliani;
    static PFNGLVERTEXATTRIB4FVPROC         glVertexAttrib4fv_Guiliani;
    static PFNGLVERTEXATTRIB4IVPROC         glVertexAttrib4iv_Guiliani;
    static PFNGLVERTEXATTRIB4SPROC          glVertexAttrib4s_Guiliani;
    static PFNGLVERTEXATTRIB4SVPROC         glVertexAttrib4sv_Guiliani;
    static PFNGLVERTEXATTRIB4UBVPROC        glVertexAttrib4ubv_Guiliani;
    static PFNGLVERTEXATTRIB4UIVPROC        glVertexAttrib4uiv_Guiliani;
    static PFNGLVERTEXATTRIB4USVPROC        glVertexAttrib4usv_Guiliani;
    static PFNGLVERTEXATTRIBPOINTERPROC     glVertexAttribPointer_Guiliani;
#endif

#endif

    /**
    Initialize function pointers to gl-functions
    */
    void InitGLProcs();

#if GUILIANI_USE_GL_SHADERS == 1
    /** Initializes default shader objects for OpenGL(ES)
    @return true if successful
    */
    eC_Bool InitShaders();

    /** Matrix multiplication.
    The matrix given in avMatrix will be multiplied with the matrix set by SetMatrix(), which will also
    be used to store the result of the multiplication.  ActiveMatrix = avMatrix * ActiveMatrix
    @param avMatrix Matrix to multiply the current matrix with.
    */
    void MultMatrix(const Matrix3D_t& avMatrix);

    ///Replacement for glOrtho in OpenGL ES 2.x
    void SetOrthoProjection(const eC_Value& vLeft, const eC_Value& vRight,
        const eC_Value& vBottom, const eC_Value& vTop,
        const eC_Value& vNear, const eC_Value& vFar);

    ///Replacement for glFrustum in OpenGL ES 2.x
    void SetFrustumProjection(const eC_Value& vLeft, const eC_Value& vRight,
        const eC_Value& vBottom, const eC_Value& vTop,
        const eC_Value& vNear, const eC_Value& vFar);

    ///Replacement for glRotate in OpenGL ES 2.x
    void SetRotation(const eC_Value& vDegrees, eC_Value vX, eC_Value vY, eC_Value vZ);

    ///Replacement for glTranslate in OpenGL ES 2.x
    void SetTranslation(const eC_Value& vXTranslate, const eC_Value& vYTranslate, const eC_Value& vZTranslate);

    ///Replacement for glScale in OpenGL ES 2.x
    void SetScaling(const eC_Value& vXScale, const eC_Value vYScale, const eC_Value vZScale);

    ///Calls SetScaling with the same parameter for the whole matrix
    void SetUniformScaling(const eC_Value& vScale);

    /** Returns the program object to get access to the shader program.
    @return Program object.
    */
    eC_UInt GetShaderID() const;

    /** Replaces the currently used matrix with ne new one.
    @param avNewMatrix New matrix to set.
    */
    void SetMatrix(const Matrix3D_t& avNewMatrix);

    /** Returns the currently active matrix (or identity matrix in default switch case).
    @return Active matrix.
    */
    const Matrix3D_t& GetMatrix() const;

    /** Pushes the currently active Matrix onto the stack. */
    void PushMatrix();

    /** Pops the last used active Matrix from stack. */
    void PopMatrix();

    /** Available matrix types */
    enum MatrixType_t
    {
        MT_PROJECTION,
        MT_MODELVIEW
    };

    /** Sets the matrix mode
    @param eNewActiveMatrix Matrix to set active.
    */
    inline void SetMatrixMode(const MatrixType_t& eNewActiveMatrix)
    {
        m_eActiveMatrix = eNewActiveMatrix;
    }

    /** Sets identity matrix to currently active one. */
    void LoadIdentity();

#endif //#if GUILIANI_USE_GL_SHADERS == 1

    /** Check if FBO is supported
    @return
    */
    eC_Bool IsFBOSupported() const { return m_bFBOSupported; }

    /** set client-sates
    @param bVertexEnabled
    @param bTextureCoordsEnabled
    */
    void SetClientStates(
        const eC_Bool& bVertexEnabled,
        const eC_Bool& bTextureCoordsEnabled);

    /** Bind buffer
    @param eType
    @param uiBuffer
    */
    void BindBuffer(const eC_UInt& eType, const eC_UInt& uiBuffer);

    /** Bind texture
    @param eType
    @param uiTextureID
    */
    void BindTexture(const eC_UInt& eType, const eC_UInt& uiTextureID);

    /** Set texturing active
    @param bTexturingActive
    */
    void SetTexturingActive(const eC_Bool& bTexturingActive);

    /** Set color-mixing
    @param bMixWithColor
    */
    void SetMixTextureWithColor(const eC_Bool& bMixWithColor);

    /** Create texture
    @return texture-id
    */
    eC_UInt CreateTexture();

    /** delete texture
    @param uiTextureID
    */
    void DeleteTexture(const eC_UInt& uiTextureID);

    /** Read data from texture
    @param uiTexID
    @param uiXPos
    @param uiYPos
    @param uiWidth
    @param uiHeight
    @param pkData
    */
    virtual void ReadTextureData(
        const eC_UInt& uiTexID,
        const eC_UInt& uiXPos, const eC_UInt& uiYPos,
        const eC_UInt& uiWidth, const eC_UInt& uiHeight, void* pkData) = 0;

    /// enable blending
    void EnableBlending();

    /// disable blending
    void DisableBlending();

protected:
    // ---------------------------------------------------------------
    // Construction / destruction
    // ---------------------------------------------------------------

    /// Destructor
    virtual ~CGfxWrapGL(void);

    /// Constructor
    CGfxWrapGL(void);

    virtual CGUIBitmapPlane* CreateGfxDependentBitmap(
        GUIBitmapPlaneID_t uiNewBitmapID,
        const CGUIObject& rkParentObject,
        const eC_Value& vWidth, const eC_Value& vHeight);

    /** Restores the Backbuffer of the display after it has been used for texture rendering
    Always begins in LOWER LEFT of screen!
    @param vWidth of the area to restore.
    @param vHeight of the area to restore.
    */
    virtual void RestoreBackBuffer(const eC_Value vWidth, const eC_Value vHeight);

    /** Loads an image.
    This loads an image from the given path and associates it with an Image-Identifier.
    @param psPath Complete path and filename of the image to be loaded.
    @param eID ID by which the image will later be accessible.
    */
    virtual void LoadImgImpl(const eC_String& psPath, const ImageResource_t &eID);

    /** Loads an image file from disk. Internally called by ResourceManager.
    @param pkMemory memory-location
    @param uiWidth
    @param uiHeight
    @param eImageType
    @param eID image-id which will be set
    */
    virtual void LoadImgImpl(
        void* pkMemory,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& eImageType,
        const ImageResource_t &eID);

    /** Unloads an image.
    Unloads the image associated with the supplied Image-identifier and frees the memory formerly
    occupied by this image.
    @param eID Identifier of the image to be unloaded.
    */
    virtual void UnloadImg(const ImageResource_t &eID);

    eC_Bool SetNOFImagesImpl(const eC_UInt uiNOFImages);

    virtual void BlitImgExtImpl(
        const ImageResource_t &eID,
        const eC_UInt &uiSrcX, const eC_UInt &uiSrcY,
        const eC_UInt &uiSrcWidth, const eC_UInt &uiSrcHeight,
        const eC_Value& vDestX, const eC_Value& vDestY,
        const eC_Value& vDestWidth, const eC_Value& vDestHeight,
        const eC_UByte &ubAlpha = 255,
        const eC_Value& vAngle = eC_FromFloat(0),
        const eC_Value& vRotCenterX = eC_FromFloat(0.5), const eC_Value& vRotCenterY = eC_FromFloat(0.5));

    virtual void BlitImgExtImpl(const CGUIImageData& kImageData, const eC_Value& vAbsDestX, const eC_Value& vAbsDestY) {}

    virtual void BlitImgNinePatch(
        const ImageResource_t &eID,
        const CGUIRect& kAbsRect,
        const eC_UInt uiTopBorder,
        const eC_UInt uiBottomBorder,
        const eC_UInt uiLeftBorder,
        const eC_UInt uiRightBorder,
        const eC_UByte &ubAlpha);

    /// helper for blitting ninepatch
    void BlitNinePatchExt();

    /** Gets the width of an image.
    @param eID Identifier of the image.
    @return The width of the image in pixels.
    */
    virtual eC_UInt GetImgWidthImpl(const ImageResource_t &eID) const;

    /** Gets the height of an image.
    @param eID Identifier of the image.
    @return The height of the image in pixels.
    */
    virtual eC_UInt GetImgHeightImpl(const ImageResource_t &eID) const;

    /** Returns the amount of memory taken up by an image.
    @param eID ID of the image to be measured.
    @return Size of the image in byte.
    */
    virtual eC_UInt GetImageSize(const ImageResource_t& eID) const;

    /** Draws a rectangle
    @param vX1 X position of corner 1
    @param vY1 Y position of corner 1
    @param vX2 X position of corner 2
    @param vY2 Y position of corner 2
    @param bFilled Whether to draw a filled rectangle
    */
    void DrawRect(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Bool bFilled);

    /** Generates a texture of matching (power of 2) size for the given image ID and copies the image into it
    @param eImageID ImageID for which the texture should be created.
    */
    void GenerateTexture(const ImageResource_t &eImageID);

    /** Accessing the Z-Correction for current render target.
    * Render targets may be the screen or any BitmapPlane.
    *  @return returns the current z-correction for the active render target.
    */
    eC_Value GetZCorrection();

    /** Increases the Z-Correction for the current render target.
    * The render target may be the screen or any BitmapPlane.
    */
    void IncreaseZCorrection();

    /** This SetCliprect version is used for an anti-clockwise rotation of the GUI. E.g. in GfxWrapEAGL implementation, if the iOS device is rotated.
    @param rkNewClipRect A normalised rectangle that includes all pixels that could be drawn
    by the functions of the graphics wrapper. The rectangle includes the
    border. All pixels outside will be left untouched.
    @param bRotate Set this true to set glScissor rotated anti-clockwise.
    */
    void SetCliprect(const CGUIRect &rkNewClipRect, const eC_Bool &bRotate);

    /** Method to initialize the graphic library
    */
    void Init(void);

    /** Method to give the graphic library the opportunity to clean up
    */
    void DeInit(void);

    /** Set scissor-area
    @param bEnabled
    @param iX
    @param iY
    @param iWidth
    @param iHeight
    */
    void SetScissor(
        const eC_Bool& bEnabled,
        const eC_Int& iX = 0,
        const eC_Int& iY = 0,
        const eC_Int& iWidth = 0,
        const eC_Int& iHeight = 0
    );

protected:
    eC_Value m_vRed; ///< red
    eC_Value m_vGreen; ///< green
    eC_Value m_vBlue; ///< blue
    eC_Value m_vAlpha; ///< alpha

    eC_TArray<CGUIImageDataGL*> m_apTextures; ///< used textures

    eC_Value m_vDepthCorrection; ///< depth-corretion

    /// transition matrix
    Matrix3D_t m_avTransMat;

    eC_Value m_vFovy; ///< fov
    eC_Value m_vAspect; ///< aspect-ratio
    eC_Value m_vNear; ///< near-plane
    eC_Value m_vFar; ///< far-plane

    /// vertex buffer object
    eC_UInt m_uiVertexVBO;

#if GUILIANI_USE_GL_SHADERS == 1
    /// projection matrix
    Matrix3D_t m_avProjectionMatrix;

    /// modelview matrix
    Matrix3D_t m_avModelviewMatrix;

    /// modelview matrix stack
    eC_TListDoubleLinked<Matrix3D_t*> m_kModelviewMatrixStack;

    /// projection matrix stack
    eC_TListDoubleLinked<Matrix3D_t*> m_kProjectionMatrixStack;

    CGfxWrapGLShader* m_pkShader;

    ///active matrix
    MatrixType_t m_eActiveMatrix;

    eC_Int m_iLocTextureEnvMode; ///< texture-env mode
    eC_Int m_iLocTexSampler; ///< texture-sampler
    eC_Int m_iLocModelViewMatrix; ///< model-view-matrix
    eC_Int m_iLocProjectionMatrix; ///< projection-matrix
    eC_Int m_iLocMix; ///< mixing-matrix
#endif //#if GUILIANI_USE_GL_SHADERS == 1

    ///< Indicates if FrameBufferObjects are supported on this machine
    eC_Bool m_bFBOSupported;

private:
    /** internal states to prevent too much changes */
    eC_Bool m_bVertexEnabled;
    eC_Bool m_bTextureCoordsEnabled;

    /** currently bound buffer to reduce buffer-switching */
    eC_UInt m_uiCurrentlyBoundBuffer;

    /** currently bound texture to reduce texture-switching */
    eC_UInt m_uiCurrentlyBoundTexture;

    /** set if texturing is currently active */
    eC_Bool m_bTexturingActive;

    struct BlitRecord_t
    {
        void Reset()
        {
            bActive = false;
        }

        eC_Bool bActive;
        eC_UInt uiSrcX;
        eC_UInt uiSrcY;
        eC_UInt uiSrcWidth;
        eC_UInt uiSrcHeight;
        eC_Value vDestX;
        eC_Value vDestY;
        eC_Value vDestWidth;
        eC_Value vDestHeight;
    };

    struct NinePatchBlit_t
    {
        void Reset()
        {
            eImageID = DUMMY_IMAGE;
            for (eC_UByte index = 0; index < 9; ++index)
                kCoords[index].Reset();
            ubAlpha = 0;
        }

        ImageResource_t eImageID;
        BlitRecord_t kCoords[9];
        eC_UByte ubAlpha;
    };

    NinePatchBlit_t m_kBlits;

    // scissoring
    eC_Bool m_bScissorEnabled;
    eC_Int m_iScissorX;
    eC_Int m_iScissorY;
    eC_Int m_iScissorWidth;
    eC_Int m_iScissorHeight;
};

#endif
