/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __EGL_HELPER_H__
#define __EGL_HELPER_H__

#include <EGL/egl.h>
#include <EGL/eglext.h>

#include "GUITrace.h"

/**
The official EGL Specification released by Khronos states in Appendix D Version 1.3
"For naming consistency, some tokens from EGL 1.2 have been renamed [...] The old names are also retained for backwards compatibility."
Unfortunately this is not implemented for some SDKs. The following definitions resolve this incompatibility.
*/
#ifdef GUILIANI_EGL13_COMPATIBILITY_WORKAROUND
#undef EGL_DEFAULT_DISPLAY
#define EGL_DEFAULT_DISPLAY     ((EGLNativeDisplayType)0)
typedef EGLNativeWindowType     NativeWindowType;
typedef EGLNativeDisplayType    NativeDisplayType;
#endif

#define GETEGLHELPER EGLHelper::GetInstance()

/** This provides a helper methods to init EGL */
class EGLHelper
{
public:
    /** Create an instance
        @param uiVersion OpenGL ES version to instantiate
        @param iID pick config with id (set to -1 if not interested) (default: -1)
        @param iBufferSize size of buffer for config (default: 16)
        @param iRedSize size of red component for config (default: 1)
        @param iGreenSize size of green component for config (default: 1)
        @param iBlueSize size of blue component for config (default: 1)
        @param iAlphaSize size of alpha component for config (default: 0)
        @param iDepthSize size of depth buffer for config (default: 16)
        @return true if successful
    */
    static eC_Bool CreateInstance(
        const eC_UInt& uiVersion,
        const eC_Int& iID = -1,
        const eC_Int& iBufferSize = 16,
        const eC_Int& iRedSize = 1,
        const eC_Int& iGreenSize = 1,
        const eC_Int& iBlueSize = 1,
        const eC_Int& iAlphaSize = 0,
        const eC_Int& iDepthSize = 16);

    /** Constructor
        @param uiVersion OpenGL ES version to instantiate
        @param iID pick config with id (set to -1 if not interested) (default: -1)
        @param iBufferSize size of buffer for config (default: 16)
        @param iRedSize size of red component for config (default: 1)
        @param iGreenSize size of green component for config (default: 1)
        @param iBlueSize size of blue component for config (default: 1)
        @param iAlphaSize size of alpha component for config (default: 0)
        @param iDepthSize size of depth buffer for config (default: 16)
    */
    EGLHelper(
        const eC_UInt& uiVersion,
        const eC_Int& iID,
        const eC_Int& iBufferSize,
        const eC_Int& iRedSize,
        const eC_Int& iGreenSize,
        const eC_Int& iBlueSize,
        const eC_Int& iAlphaSize,
        const eC_Int& iDepthSize);

    /** Destructor */
    virtual ~EGLHelper();

    /** get instance
        @return instance
    */
    static EGLHelper& GetInstance();

    /** Checks for EGL error and prints the error code.
        @param[in] kCallingMethod Method name which calls this function. Used to print the method name.
        @return True if there is an error, False otherwise
    */
    static eC_Bool CheckEGLError(const eC_String &kCallingMethod);

    /** swap buffers
        @return true if successful
    */
    virtual eC_Bool SwapBuffers();

    /** Returns the screen context.
        @return Screen context
    */
    inline const EGLContext GetScreenContext() const
    {
        return m_pkEGLScreenContext;
    }

    /** Returns the offscreen context.
        @return Offscreen context
    */
    inline const EGLContext GetOffScreenContext() const
    {
        return m_pkEGLOffScreenContext;
    }

    /** Returns the EGL display handle.
        @return EGL display handle
    */
    inline const EGLDisplay GetDisplay() const
    {
        return m_pkEGLDisplay;
    }

    /** Returns the screen config.
        @return Screen config
    */
    inline const EGLConfig GetScreenConfig() const
    {
        return m_pkScreenConfig;
    }

    /** Returns the offscreen config.
        @return Offscreen config
    */
    inline const EGLConfig GetOffScreenConfig() const
    {
        return m_pkOffScreenConfig;
    }

    /** Returns the EGL window surface.
        @return EGL window surface
    */
    inline const EGLSurface GetWindowSurface() const
    {
        return m_pkEGLWindowSurface;
    }

    /** Return if EGL is inited
        @return true if successfully inited
    */
    eC_Bool IsInited() const
    {
        return m_bInited;
    }

    /** Return width of EGL-screen
        @return width
    */
    eC_UInt GetWidth() const;

    /** Return height of EGL-screen
        @return height
    */
    eC_UInt GetHeight() const;

private:
    /** default constructor
    */
    EGLHelper();

    /** Initialises the EGL surface.
        @return True if OK, False otherwise
    */
    eC_Bool Init();

    EGLBoolean CreateEGLContext();

    /** Deinitialises the EGL surface.
        @return True if OK, False otherwise
    */
    eC_Bool DeInit();

    /** Helper method. */
    eC_Bool FindScreenConfig();

    /** Helper method. */
    eC_Bool FindOffScreenConfig();

    void PrintConfig(EGLDisplay pkDisplay, EGLConfig config);

private:
    /** The screen buffer size. Total depth of the color buffer in bits.
        Feel free to set this as required BEFORE calling Createinstance() on derived graphic wrappers. */
    static EGLint ms_uiEGLScreenBufferSize;

    /** The offscreen buffer size. Total depth of the offscreen color buffer in bits.
        Feel free to set this as required BEFORE calling Createinstance() on derived graphic wrappers. */
    static EGLint ms_uiEGLOffScreenBufferSize;

    /** This allows the definition of custom attributes for the screen configuration.
        The given array of attributes will be handed over to eglChooseConfig during initialization.
        @code
            // Example custom configuration
            EGLint myconfig[] = {
                    EGL_BUFFER_SIZE, 32,
                    EGL_SURFACE_TYPE, EGL_PBUFFER_BIT,
                    EGL_RENDERABLE_TYPE, EGL_OPENGL_ES_BIT,
                    EGL_NONE
                };
            // Hand over custom configuration to CEGLEnvironment
            CEGLEnvironment::m_aiScreenConfig = myconfig;
            // Finally create instance of derived graphics wrapper
            CGfxWrapOGLES1_1::CreateInstance((NativeWindowType)hWnd, (NativeDisplayType) GetDC(hWnd));
        @endcode
        Feel free to set this as required BEFORE calling Createinstance() on derived graphic wrappers. */
    static EGLint* m_aiScreenConfig;

    /** This allows the definition of custom attributes for the OffScreen configuration.
        The given array of attributes will be handed over to eglChooseConfig during initialization.
        Feel free to set this as required BEFORE calling Createinstance() on derived graphic wrappers. */
    static EGLint* m_aiOffScreenConfig;

    /// our EGL display
    EGLDisplay m_pkEGLDisplay;

    /// our EGL surface
    EGLSurface m_pkEGLWindowSurface;

    /// our EGL screen context
    EGLContext m_pkEGLScreenContext;

    /// our EGL offscreen context used for offscreen rendering
    EGLContext m_pkEGLOffScreenContext;

    /// our EGL configuration for the screen buffer
    EGLConfig m_pkScreenConfig;

    /// our EGL configuration for the offscreen buffer
    EGLConfig m_pkOffScreenConfig;

    /// the EGL major version
    EGLint m_iEGLMajorVersion;

    /// the EGL minor version
    EGLint m_iEGLMinorVersion;

    /// version set by constructor parameters. Used to receive the correct EGLconfig
    eC_UInt m_uiVersion;

    /// Sets the wanted renderable type by given version number
    EGLint m_iRenderableType;

    /// the display handle
    NativeDisplayType m_hDisplayType;

    /// the window handle
    NativeWindowType m_hWindowType;

    /// use platform displays (GBM, Wayland, etc.)
    eC_Bool m_bUsePlatformDisplay;

    static EGLHelper* ms_pkInstance;

    /// settings to choose config
    EGLint m_iID;
    EGLint m_iBufferSize;
    EGLint m_iRedSize;
    EGLint m_iGreenSize;
    EGLint m_iBlueSize;
    EGLint m_iAlphaSize;
    EGLint m_iDepthSize;

    eC_Bool m_bInited;
};

#endif
