/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GFXWRAPEGML__H_
#define GFXWRAPEGML__H_

#include "GfxWrap.h"
#include "eGML_Config.h"
#include "eGML_Platform.h"
#include "eGML_Bitmap.h"

class eGML;
class eImg_Bitmap;
class eGML_Screen;
class eGML_Bitmap;
class CDCWrap;

///  This is the eGML implementation of the graphics wrapper.

/** This graphics wrapper uses eGML_Bitmap methods for blitting. The blit flags
for all operations can be set by calling SetBlitFlags(). They are by default
initialized to eGML_Bitmap::BLIT_BILINEAR.

For additional details see description of base class CGfxWrap.
*/
class CGfxWrapeGML : public CGfxWrap
{
public:
    /** Singleton for creating a GfxWrapeGML object and change the pointer in
    base class to this graphics wrapper.
    @param pkGfxEnv graphics-environment
    @return Indicate whether by this call, an instance is created.
    The instance is created, if true, not created, otherwise.
    */
    static eC_Bool CreateInstance(CGfxEnv* pkGfxEnv);

    virtual eC_Value SetLineWidth(const eC_Value& vWidth);

    virtual eC_Bool Refresh(const RefreshCall_t eRefreshCallType);

    /** create a screen with given height and attributes
    */
    virtual void CreateScreen();

    virtual void DestroyScreen();

    ///@return the eGML screen pointer
    eGML_Screen* GetScreen(void);

    ///@return the eGML render bitmap pointer
    eGML_Bitmap* GetRenderBitmap();

    ///@param pRenderBitmap The target bitmap for subsequent render operations
    void SetRenderBitmap(eGML_Bitmap* pRenderBitmap);

    ///@param uiLayerID Switch to buffer for layer for subsequent render operations
    void SetTargetLayer(const eC_UInt& uiLayerID);

    virtual eC_UInt GetVirtualScreenWidth();
    virtual eC_UInt GetVirtualScreenHeight();
    virtual eC_UInt GetPhysicalScreenWidth();
    virtual eC_UInt GetPhysicalScreenHeight();

    virtual void SetForegroundColorImpl(
        const eC_UByte &ubRed,
        const eC_UByte &ubGreen,
        const eC_UByte &ubBlue,
        const eC_UByte &ubAlpha = 255);

    virtual void Line(const eC_Value& vX1, const eC_Value& vY1, const eC_Value& vX2, const eC_Value& vY2);

    virtual void Rect(const eC_Value& vX1, const eC_Value& vY1, const eC_Value& vX2, const eC_Value& vY2);

    virtual void FilledRect(const eC_Value& vX1, const eC_Value& vY1, const eC_Value& vX2, const eC_Value& vY2);

    /** Implementation of the CGfxWrap::Ellipse() interface for eGML.
    Note that eGML does currently only support filled circles (e.g. width and height must be identical), and
    the active foreground color must be fully opaque.
    @param vAbsX1 X-coordinate of bounding rectangle's top left corner
    @param vAbsY1 Y-Coordinate of bounding rectangle's top left corner
    @param vAbsX2 X-coordinate of bounding rectangle's bottom right corner
    @param vAbsY2 Y-Coordinate of bounding rectangle's bottom right corner
    @param bFilled Specifies whether the ellipse will be drawn outlined (false) or filled (true).
    */
    virtual void Ellipse(const eC_Value& vAbsX1, const eC_Value& vAbsY1, const eC_Value& vAbsX2, const eC_Value& vAbsY2, const eC_Bool &bFilled);

    virtual void Arc(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Value& vStartAngle,
        const eC_Value& vEndAngle,
        const eC_Value& vRotationAngle,
        const eC_Bool &bFilled);

    virtual void Ring(
        const eC_Value& vX1,
        const eC_Value& vY1,
        const eC_Value& vX2,
        const eC_Value& vY2,
        const eC_Value& vRingWidth,
        const eC_Value& vStartAngle,
        const eC_Value& vEndAngle,
        const eC_Value& vRotationAngle,
        const eC_Bool & bFilled);

    virtual void Polygon(
        eC_Value* pvPoints,
        const eC_UInt& uiCount,
        const eC_Bool& bFilled);

    virtual void SetCliprect(const CGUIRect &NewAbsClipRect);

    /** Returns a pointer to the image referenced by the specified ID.
    @param eID of the image to retrieve
    @return eImb_Bitmap pointer to the related bitmap   */
    virtual eImg_Bitmap* GetImage(const ImageResource_t &eID);

    /** Returns whether an image is present for the given ID.
    @param eID ID for which to check
    @return True if an image is present, False otherwise. */
    virtual eC_Bool ImageExists(const ImageResource_t eID) const;

    /** Enables/disables subpixel rendering.  (Disabled by default!)
    While subpixel rendering is enabled, blitting will be done with subpixel-accuracy (Resulting in a time-consuming RotBlit)
    @param bEnabled true to enable subpixel rendering, false to disable. */
    void SetSubpixelEnabled(const eC_Bool bEnabled);

    /** Sets the eGML blit flags for all blit operations.
    @param uiFlags Combination of eGML blit flags.
    @see GetBlitFlags()
    */
    void SetBlitFlags(eC_Flag uiFlags);

    /** Returns the currently set eGML blit flags.
    @return The current combination of eGML blit flags.
    @see SetBlitFlags()
    */
    eC_Flag GetBlitFlags() const;

    /** Enables/Disables NEON optimization
    if neon is available this setting is true by default
    @param bOptimizedBlit true to enable NEON optimization if available, false to disable
    */
    virtual void SetOptimizedBlit(const eC_Bool& bOptimizedBlit);

    /** Enables/Disables bilinear filtering during blit operation
    if bilinear is available this setting is true by default
    @param bFilteredBlit true to enable bilinear blit, false to disable bilinear filtering and use nearest neighbor texturing
    */
    virtual void SetFilteredBlit(const eC_Bool& bFilteredBlit);

    virtual eC_Bool DrawToScreen();

protected:
    /** Sets the total number of images. This is called internally by the ResourceManager when dynamically
    allocated image resources require the total number of images to be increased.
    @param uiNOFImages The number of images
    @return True if internal storage has been successfully adapted to support the requested
    number of images. False otherwise.
    */
    virtual eC_Bool SetNOFImagesImpl(const eC_UInt uiNOFImages);

    /** Loads an image file from disk. Internally called by ResourceManager.
    @param psPath filename of image to load
    @param eID image-id which will be set
    */
    virtual void LoadImgImpl(const eC_String &psPath, const ImageResource_t &eID);

    /** Loads an image file from disk. Internally called by ResourceManager.
    @param pkMemory memory-location
    @param uiWidth
    @param uiHeight
    @param eImageType
    @param eID image-id which will be set
    */
    virtual void LoadImgImpl(
        void* pkMemory,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& eImageType,
        const ImageResource_t &eID);

    /** Unloads an image file. Internally called by ResourceManager.
    @param eID image-id to unload
    */
    virtual void UnloadImg(const ImageResource_t &eID);

    virtual void BlitImgExtImpl(
        const ImageResource_t &eID,
        const eC_UInt &uiSrcX, const eC_UInt &uiSrcY,
        const eC_UInt &uiSrcWidth, const eC_UInt &uiSrcHeight,
        const eC_Value& vDestX, const eC_Value& vDestY,
        const eC_Value& vDestWidth, const eC_Value& vDestHeight,
        const eC_UByte &ubAlpha = 255,
        const eC_Value& vAngle = eC_FromFloat(0),
        const eC_Value& vRotCenterX = eC_FromFloat(0.5), const eC_Value& vRotCenterY = eC_FromFloat(0.5));

    virtual void BlitImgExtImpl(const CGUIImageData& kImageData, const eC_Value& vAbsDestX, const eC_Value& vAbsDestY);

    virtual eC_UInt GetImgWidthImpl(const ImageResource_t &eID) const;
    virtual eC_UInt GetImgHeightImpl(const ImageResource_t &eID) const;

    /** Calculates the size of an image in pixels. Called by the image cache.
    @param eID id of image
    @return size of image in pixels
    */
    virtual eC_UInt GetImageSize(const ImageResource_t& eID) const;

    /** Sets the default 2D view used by the gfx wrapper for 2D drawing operations, like FilledRect, Line, Blit, etc.
    */
    void SetDefault2DView();

    /** Initialize the main-layer
    */
    virtual void InitMainLayer();

    /** Constructor
    @param pkGfxEnv
    */
    CGfxWrapeGML(CGfxEnv* pkGfxEnv);

    /** Destructor
    */
    virtual ~CGfxWrapeGML(void);

protected:
    /** Screen-configuration to be used in GfxWrapper
    */
    typedef struct
    {
        eC_UInt uiID;           ///< id of screen
        eC_UInt uiScreenCount;  ///< number of screens
        eGML_Screen* pkScreen;  ///< pointer to screens
    } ScreenConfig_t;

    eGML* m_pkEGML; ///< eGML instance pointer
    eGML_Bitmap* m_pkRenderBitmap;  ///< Target bitmap for render operations in eGML
    ScreenConfig_t* m_pkScreens; ///< Screens for eGML

    eC_UInt m_uiNumberOfScreens; ///< number of configured screens
    eC_Int m_iMainLayerIndex; ///< index of the main-screen

    /// while subpixel rendering is enabled, blitting will be done with subpixel-accuracy (Resulting in a time-consuming RotBlit)
    eC_Bool m_bSubpixelEnabled;

    /// Array of pointers to eImg bitmaps (includes alpha layer)
    eC_TArray <eImg_Bitmap*> m_pImage;

    eC_Flag m_uiBlitFlags;  ///<Flags for blit operation

private:
    class CModifyableBitmap : public eGML_Bitmap
    {
    public:
        CModifyableBitmap(eC_UInt uiWidth, eC_UInt uiHeight, eC_UByte ubBitmapBufferType) :
            eGML_Bitmap(uiWidth, uiHeight, ubBitmapBufferType)
        {
        }

        ~CModifyableBitmap()
        {
        }

        /** Initialize this instance with new image information.
            @param eFormat Format of Bitmap (see eGML_Config.h)
            @param uiWidth Width of bitmap data in pixels
            @param uiHeight Height of bitmap data in pixels
            @param ubBitmapBufferType eGML bitmap buffertype (see eGML_Config.h)
            @param pBitmapBuffer Pointer to raw bitmap pixel data
        */
        void SetNewBitmapData(eC_UInt eFormat, eC_UInt uiWidth, eC_UInt uiHeight, eC_UByte ubBitmapBufferType, void *pBitmapBuffer)
        {
            m_uiType = eFormat;
            m_uiWidth = uiWidth;
            m_uiHeight = uiHeight;
            m_ubBBT = ubBitmapBufferType;
            m_pkData = pBitmapBuffer;

            // pitch depends on bitmap format
            switch (eFormat)
            {
                case eGML::BM_ALPHA1:
                    m_uiPitch = (uiWidth + 7) & ~0x07;
                    break;
                case eGML::BM_ALPHA8:
                    m_uiPitch = uiWidth;
                    break;
                case eGML::BM_RGB565:
                case eGML::BM_RGB555:
                    m_uiPitch = uiWidth * 2;
                    break;
                case eGML::BM_RGB32:
                case eGML::BM_ARGB32:
                    m_uiPitch = uiWidth * 4;
                    break;
                default:
                    GUILOG(GUI_TRACE_ERROR, "Unknown bitmap format. Failed to determine pitch.\n");
                    break;
            }
        }

        virtual void * Lock()
        {
            // Dummy implementation of pure-virtual function in base-class
            return NULL;
        }

        virtual void Unlock(eC_UInt uiX1 = 0, eC_UInt uiY1 = 0, eC_UInt uiX2 = 0, eC_UInt uiY2 = 0)
        {
            // Dummy implementation of pure-virtual function in base-class
        }

        virtual void DrawLine(const eGML_Vector2 &kStart, const eGML_Vector2 &kEnd, eGML_Color kColor, eC_Flag ulFlags, eC_Value vStartWidth, eC_Value vEndWidth = 0)
        {
            // Dummy implementation of pure-virtual function in base-class
        }

        virtual void DrawPolygon(const eGML_Coord2D *pkVertices, eC_UInt uiCount, eGML_Color kColor, eC_Flag ulFlags)
        {
            // Dummy implementation of pure-virtual function in base-class
        }

        virtual void Clear(eGML_Color clColor)
        {
            // Dummy implementation of pure-virtual function in base-class
        }

        virtual void FillRect(eC_UInt uiWidth, eC_UInt uiHeight, eC_Int iX1, eC_Int iY1, eGML_Color clColor, eC_Flag ulFlags = 0)
        {
            // Dummy implementation of pure-virtual function in base-class
        }
    };

private:
    virtual CGUIBitmapPlane* CreateGfxDependentBitmap(
        GUIBitmapPlaneID_t uiNewBitmapID,
        const CGUIObject& rkParentObject,
        const eC_Value& vWidth,
        const eC_Value& vHeight);

    eC_Bool Init(void);
    eC_Bool DeInit(void);

    eC_UInt GetBitmapFormat(const CGUICommonEnums::ImageType_t& ePixFormat) const;

    void CreateScreen(
        const eC_UInt& uiIndex, const eC_UInt& uiLayerID,
        const eC_Enum& eBitmapFormat,
        const eC_UInt& uiWidth, const eC_UInt& uiHeight,
        const eC_UInt& ubBufferCount,
        void** pkBuffers);

    void MergeLayer(const eC_UInt& uiScreenIndex);

    /** Copy-constructor intentionally not implemented.
    */
    CGfxWrapeGML(const CGfxWrapeGML& kSource);

    /** Operator= method intentionally not implemented.
    */
    CGfxWrapeGML& operator=(const CGfxWrapeGML& kSource);

private:
    CModifyableBitmap* m_pkModifyableGlyphBitmap;
};

#endif
