/***********************************************************************************************************************
 * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
 *
 * This software and documentation are supplied by Renesas Electronics Corporation and/or its affiliates and may only
 * be used with products of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.
 * Renesas products are sold pursuant to Renesas terms and conditions of sale.  Purchasers are solely responsible for
 * the selection and use of Renesas products and Renesas assumes no liability.  No license, express or implied, to any
 * intellectual property right is granted by Renesas.  This software is protected under all applicable laws, including
 * copyright laws. Renesas reserves the right to change or discontinue this software and/or this documentation.
 * THE SOFTWARE AND DOCUMENTATION IS DELIVERED TO YOU "AS IS," AND RENESAS MAKES NO REPRESENTATIONS OR WARRANTIES, AND
 * TO THE FULLEST EXTENT PERMISSIBLE UNDER APPLICABLE LAW, DISCLAIMS ALL WARRANTIES, WHETHER EXPLICITLY OR IMPLICITLY,
 * INCLUDING WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NONINFRINGEMENT, WITH RESPECT TO THE
 * SOFTWARE OR DOCUMENTATION.  RENESAS SHALL HAVE NO LIABILITY ARISING OUT OF ANY SECURITY VULNERABILITY OR BREACH.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT WILL RENESAS BE LIABLE TO YOU IN CONNECTION WITH THE SOFTWARE OR
 * DOCUMENTATION (OR ANY PERSON OR ENTITY CLAIMING RIGHTS DERIVED FROM YOU) FOR ANY LOSS, DAMAGES, OR CLAIMS WHATSOEVER,
 * INCLUDING, WITHOUT LIMITATION, ANY DIRECT, CONSEQUENTIAL, SPECIAL, INDIRECT, PUNITIVE, OR INCIDENTAL DAMAGES; ANY
 * LOST PROFITS, OTHER ECONOMIC DAMAGE, PROPERTY DAMAGE, OR PERSONAL INJURY; AND EVEN IF RENESAS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH LOSS, DAMAGES, CLAIMS OR COSTS.
 **********************************************************************************************************************/

#include "rza3ul_smarc_lcd.h"
#include "FreeRTOS.h"
#include "task.h"
#include "event_groups.h"

#define USE_ENEVT_GROUP (1)

static volatile uint32_t vsync_count = 0;
static volatile uint32_t draw_buffer_index = 0;
static EventGroupHandle_t  lcd_event;
volatile uint8_t *RZAFrameBuffers[] =
{
     fb_background[0],
     fb_background[1]
};

/**
 * @name       vsync_callback
 * @brief      vsync interrupt callback
 * @param[in]  p_args
 * @return     None
 **/
void vsync_callback (display_callback_args_t * p_args)
{
    FSP_PARAMETER_NOT_USED(p_args);

    if (vsync_count > 0)
    {
#if (USE_ENEVT_GROUP)
        BaseType_t xHigherPriorityTaskWoken, xResult;

        /* xHigherPriorityTaskWoken must be initialised to pdFALSE. */
        xHigherPriorityTaskWoken = pdFALSE;

        /* Set bit 0 and bit 4 in xEventGroup. */
        xResult = xEventGroupSetBitsFromISR(
                              lcd_event,   /* The event group being updated. */
                              1, /* The bits being set. */
                              &xHigherPriorityTaskWoken );

#endif /* USE_ENEVT_GROUP */
        vsync_count--;
    }
}

/**
 * @name       wait_vsync
 * @brief      vsync wait
 * @param[in]  vsync count
 * @return     None
 **/
static void wait_vsync(uint32_t wait_count)
{
    vsync_count = wait_count;
    while (vsync_count > 0)
    {
#if (USE_ENEVT_GROUP)
        xEventGroupWaitBits(lcd_event,1,pdTRUE,pdTRUE,100);
#else
        vTaskDelay(1);
#endif /* USE_ENEVT_GROUP */
    }
}

/**
 * @name       bsp_lcd_init
 * @brief      Initialize screen rotation and open GLCDC module
 * @param[in]  None
 * @return     None
 **/
void bsp_lcd_init (void)
{
#if 0 // FULL HD (1920x1080)
    /* PLL5 Pixel Clock */
    R_CPG->CPG_SIPLL5_CLK3 = 0x40000006;
    R_CPG->CPG_SIPLL5_CLK4 = 0x004a0000;
    R_CPG->CPG_PL5_SDIV  = 0x01010202;
    R_CPG->CPG_SIPLL5_STBY = 0x00150001;
    R_CPG->CPG_RST_DDR   = 0x007F007F;
#else
    /* 74.250MHz 1280x720 */
    uint32_t reg;
    uint32_t fracin  = 10485760;
    uint32_t intin   = 86;
    uint32_t refdiv  = 1;
    uint32_t posdiv1 = 7;
    uint32_t posdiv2 = 2;
    uint32_t diva = 1;
    uint32_t divb = 0;
    R_CPG->CPG_SIPLL5_CLK3_b.FRACIN = fracin;
    R_CPG->CPG_SIPLL5_CLK4_b.INTIN = intin;
    reg = R_CPG->CPG_SIPLL5_CLK1;
    reg &= 0xFFFF0000;
    reg |= ( (1 << 24) | (1 << 20) | (1 << 16) | (refdiv << 8) | (posdiv2 << 4) | posdiv1 );
    R_CPG->CPG_SIPLL5_CLK1 = reg;
    R_CPG->CPG_PL5_SDIV    = 0x0101 | divb << 8 | diva;
    R_CPG->CPG_SIPLL5_STBY = 0x00150001;
    R_CPG->CPG_RST_DDR   = 0x007F007F;
#endif
#if (USE_ENEVT_GROUP)
    lcd_event = xEventGroupCreate();
#endif /* USE_ENEVT_GROUP */

    // Open LCDC module
    g_display0.p_api->open(g_display0.p_ctrl, g_display0.p_cfg);
}

/**
 * @name       bsp_lcd_display_on
 * @brief      Start Display
 * @param[in]  None
 * @return     None
 **/
void bsp_lcd_display_on (void)
{
    // Start LCDC module
    g_display0.p_api->start(g_display0.p_ctrl);
}

 /**
 * @name       bsp_lcd_display_off
 * @brief      Stop Display
 * @param[in]  None
 * @return     None
 **/
void bsp_lcd_display_off (void)
{
    // Stop GLCDC module
    g_display0.p_api->stop(g_display0.p_ctrl);
}

/**
* @name       bsp_lcd_display_setframeBuffer
* @brief      Change buffer
* @param[in]  Not use
* @param[in]  Not use
* @param[in]  Not use
* @return     Free buffer address
**/
void* bsp_lcd_display_setframeBuffer(void* ptr, unsigned long uiWidth, unsigned long uiHeight)
{
    void *free_buffer_address;
    FSP_PARAMETER_NOT_USED(ptr);
    FSP_PARAMETER_NOT_USED(uiWidth);
    FSP_PARAMETER_NOT_USED(uiHeight);

    free_buffer_address = (void *)RZAFrameBuffers[draw_buffer_index];
    if (draw_buffer_index == 1)
    {
        draw_buffer_index = 0;
    }
    else
    {
        draw_buffer_index = 1;
    }
    R_BSP_CACHE_CleanInvalidateRange(free_buffer_address, DISPLAY_BUFFER_STRIDE_BYTES_INPUT0 * DISPLAY_VSIZE_INPUT0);
    g_display_on_lcdc.bufferChange(
            g_display0.p_ctrl,
            (uint8_t *)ptr,
            DISPLAY_FRAME_LAYER_1
    );
    wait_vsync(1);
    return free_buffer_address;
}
